﻿
namespace Masterpiece.Controls.OutlookBar
{
    public class OutlookBarItemCollection : System.Collections.ObjectModel.ObservableCollection<OutlookBarItem> { }
    
    internal enum Direction 
    {
        Left,
        Right
    }
    [System.Windows.TemplatePart(Name = RightThumbName, Type = typeof(System.Windows.Controls.Primitives.Thumb))]
    [System.Windows.TemplatePart(Name = LeftThumbName, Type = typeof(System.Windows.Controls.Primitives.Thumb))]
    [System.Windows.TemplatePart(Name = PART_OutlookBarRoot, Type = typeof(System.Windows.Controls.Grid))]
    [System.Windows.Markup.ContentProperty("OutlookBarItems")]
    public class OutlookBar : System.Windows.Controls.Control
    {
        #region Constructors
        static OutlookBar()
        {
            System.Type ownerType = typeof(OutlookBar);

            OutlookBar.DefaultStyleKeyProperty.OverrideMetadata(ownerType, new System.Windows.FrameworkPropertyMetadata(ownerType));
        }
        public OutlookBar()
            : base()
        {
            this.OutlookBarItems = new OutlookBarItemCollection();
            this.CommandBindings.Add(new System.Windows.Input.CommandBinding(OutlookBar.ExpandCommand, new System.Windows.Input.ExecutedRoutedEventHandler(this.ExpandCommandExecuted)));

            OutlookBar.OutlookBarItemClicked += new System.EventHandler(this.OutlookBarItemClickedEventRaised);
        }
        #endregion

        #region Properties and Depenency Properties


        [System.ComponentModel.Browsable(false),
        System.ComponentModel.Bindable(true),
        System.ComponentModel.ReadOnly(false)]
        public bool IsExpanded
        {
            get { return (bool)GetValue(IsExpandedProperty); }
            set { SetValue(IsExpandedProperty, value); }
        }

        // Using a DependencyProperty as the backing store for IsExpanded.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty IsExpandedProperty =
            System.Windows.DependencyProperty.Register("IsExpanded", typeof(bool), typeof(OutlookBar), new System.Windows.PropertyMetadata(true, new System.Windows.PropertyChangedCallback(OutlookBar.OnIsExpandedPropertyChanged)));

        internal Direction Direction
        {
            get { return (Direction)GetValue(DirectionProperty); }
            set { SetValue(DirectionProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Direction.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty DirectionProperty =
            System.Windows.DependencyProperty.Register("Direction", typeof(Direction), typeof(OutlookBar), new System.Windows.PropertyMetadata(Direction.Right));

        [System.ComponentModel.Browsable(true),
       System.ComponentModel.Bindable(true),
       System.ComponentModel.ReadOnly(false)]
        public string NavigationPane
        {
            get { return (string)GetValue(NavigationPaneProperty); }
            set { SetValue(NavigationPaneProperty, value); }
        }

        // Using a DependencyProperty as the backing store for NavigationPane.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty NavigationPaneProperty =
            System.Windows.DependencyProperty.Register("NavigationPane", typeof(string), typeof(OutlookBar), new System.Windows.PropertyMetadata("Navigation Pane"));


        [System.ComponentModel.Browsable(false),
        System.ComponentModel.Bindable(true),
        System.ComponentModel.ReadOnly(false)]
        public OutlookBarItemCollection OutlookBarItems
        {
            get { return (OutlookBarItemCollection)GetValue(OutlookBarItemsProperty); }
            set { SetValue(OutlookBarItemsProperty, value); }
        }

        // Using a DependencyProperty as the backing store for OutlookBarItems.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty OutlookBarItemsProperty =
            System.Windows.DependencyProperty.Register("OutlookBarItems", typeof(OutlookBarItemCollection), typeof(OutlookBar), new System.Windows.PropertyMetadata((OutlookBarItemCollection)null));

        public int SelectedIndex
        {
            get { return (int)GetValue(SelectedIndexProperty); }
            set { SetValue(SelectedIndexProperty, value); }
        }

        // Using a DependencyProperty as the backing store for SelectedIndex.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty SelectedIndexProperty =
            System.Windows.DependencyProperty.Register("SelectedIndex", typeof(int), typeof(OutlookBar), new System.Windows.PropertyMetadata(-1, new System.Windows.PropertyChangedCallback(OutlookBar.OnSelectedIndexPropertyChanged)));

        public OutlookBarItem SelectedItem
        {
            get { return (OutlookBarItem)GetValue(SelectedItemProperty); }
            set { SetValue(SelectedItemProperty, value); }
        }

        // Using a DependencyProperty as the backing store for SelectedItem.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty SelectedItemProperty =
            System.Windows.DependencyProperty.Register("SelectedItem", typeof(OutlookBarItem), typeof(OutlookBar), new System.Windows.PropertyMetadata((OutlookBarItem)null));

        internal double PreviousWidth
        {
            get { return (double)GetValue(PreviousWidthProperty); }
            set { SetValue(PreviousWidthProperty, value); }
        }

        // Using a DependencyProperty as the backing store for PreviousWidth.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty PreviousWidthProperty =
            System.Windows.DependencyProperty.Register("PreviousWidth", typeof(double), typeof(OutlookBar), new System.Windows.PropertyMetadata(250.0D));

        [System.ComponentModel.Bindable(true),
        System.ComponentModel.Browsable(false),
        System.ComponentModel.ReadOnly(false)]
        public bool IsDropDownOpen
        {
            get { return (bool)GetValue(IsDropDownOpenProperty); }
            set { SetValue(IsDropDownOpenProperty, value); }
        }

        // Using a DependencyProperty as the backing store for IsDropDownOpen.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty IsDropDownOpenProperty =
            System.Windows.DependencyProperty.Register("IsDropDownOpen", typeof(bool), typeof(OutlookBar), new System.Windows.FrameworkPropertyMetadata(false, 
                new System.Windows.PropertyChangedCallback(OutlookBar.OnIsDropDownOpenPropertyChanged)));


        public System.Windows.Visibility ItemsVisibility
        {
            get { return (System.Windows.Visibility)GetValue(ItemsVisibilityProperty); }
            set { SetValue(ItemsVisibilityProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ItemsVisibility.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty ItemsVisibilityProperty =
            System.Windows.DependencyProperty.Register("ItemsVisibility", typeof(System.Windows.Visibility), typeof(OutlookBar), new System.Windows.PropertyMetadata(System.Windows.Visibility.Visible));

                
        #endregion

        #region Depenency Property Changed Callbacks

        private static void OnIsDropDownOpenPropertyChanged(System.Windows.DependencyObject d, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
            bool newValue = (bool)e.NewValue;
            OutlookBar outlookBar = (OutlookBar)d;
            outlookBar.SetIsOpenValue(newValue);
        }
       
        private static void OnSelectedIndexPropertyChanged(System.Windows.DependencyObject d, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
            int index = (int)e.NewValue;
            OutlookBar outlookBar = (OutlookBar)d;
            outlookBar.SelectItemByIndex(index);
        }

        private static void OnIsExpandedPropertyChanged(System.Windows.DependencyObject d, System.Windows.DependencyPropertyChangedEventArgs e)
        {
            OutlookBar outlookBar = (OutlookBar)d;
            bool newValue = (bool)e.NewValue;
            if (outlookBar.RootGrid == null) 
            {
                return;
            }
            double width = newValue ? outlookBar.PreviousWidth : 40.0D;
            outlookBar.RootGrid.SetValue(System.Windows.Controls.Grid.WidthProperty, width);
        }

        #endregion

        #region Overrides

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            if (this.RightThumb != null)
            {
                this.RightThumb.DragDelta -= this.ThumbDragDelta;
            }

            this.RightThumb = this.GetTemplateChild(OutlookBar.RightThumbName) as System.Windows.Controls.Primitives.Thumb;
            this.LeftThumb = this.GetTemplateChild(OutlookBar.LeftThumbName) as System.Windows.Controls.Primitives.Thumb;
            this.HorizontalThumb = this.GetTemplateChild(OutlookBar.HorizontalThumbName) as System.Windows.Controls.Primitives.Thumb;
            this.ItemsGrid = this.GetTemplateChild(OutlookBar.ItemsGridName) as System.Windows.Controls.Grid;
            if (this.HorizontalThumb != null && this.ItemsGrid != null)
            {
                this.HorizontalThumb.DragDelta += new System.Windows.Controls.Primitives.DragDeltaEventHandler(this.HorizontalThumbDragDelta);
            }
            this.RootGrid = this.GetTemplateChild(OutlookBar.PART_OutlookBarRoot) as System.Windows.Controls.Grid;
            if (this.RootGrid != null && !this.IsExpanded) 
            {
                this.RootGrid.SetValue(System.Windows.Controls.Grid.WidthProperty, 40.0D);
            }
            if (this.RightThumb != null && this.RootGrid != null)
            {
                this.RightThumb.DragDelta += new System.Windows.Controls.Primitives.DragDeltaEventHandler(this.ThumbDragDelta);
                this.LeftThumb.DragDelta += new System.Windows.Controls.Primitives.DragDeltaEventHandler(this.ThumbDragDelta);
            }
            this.ItemsStackPanel = this.GetTemplateChild("PART_OutlookBarItemsPanel") as System.Windows.Controls.StackPanel;
            if (this.ItemsStackPanel != null)
            {
                foreach (OutlookBarItem item in this.OutlookBarItems)
                {
                    this.ItemsStackPanel.Children.Add(item);
                }
                this.SelectItemByIndex(this.SelectedIndex);
            }
            this.LoweredItemsPanel = this.GetTemplateChild(PART_OutlookBarLoweredItemsPanelName) as System.Windows.Controls.StackPanel;
            if (this.Popup != null) 
            {
                this.Popup.Closed -= this.PopupDissmissEventRaised;
            }
            this.Popup = this.GetTemplateChild(OutlookBar.PART_OutlookBar_PopupName) as System.Windows.Controls.Primitives.Popup;
            if (this.Popup != null) 
            {
                this.Popup.Closed += new System.EventHandler(this.PopupDissmissEventRaised);
            }
            this.ContentGrid = this.GetTemplateChild(OutlookBar.PART_OutlookBarContentGridName) as System.Windows.Controls.Grid;
            
            this.PopupContentGrid = this.GetTemplateChild(OutlookBar.PART_OutlookBar_PopupContentGridName) as System.Windows.Controls.Grid;
            this.ContentPresenter = this.GetTemplateChild(OutlookBar.PART_OutlookBarContentPresenterName) as System.Windows.Controls.ContentPresenter;

        }

        protected virtual void OnMimized(object sender, System.Windows.DependencyPropertyChangedEventArgs e) { }
       
        #endregion

        #region Private Utilities

        internal static void RaiseOutlookBarItemClicked(OutlookBarItem selectedItem) 
        {
            if (OutlookBar.OutlookBarItemClicked != null) 
            {
                OutlookBarItemClicked(selectedItem, OutlookBar.privateArgs);
            }
        }

        private void SetIsOpenValue(bool value) 
        {
            if (this.Popup != null) 
            {
                if (value) 
                {
                    this.SetContentToPopup();
                }
                this.Popup.IsOpen = value;
                if (!value) 
                {
                    this.SetContentToGrid();
                }
            }
        }

        private void SetContentToPopup() 
        {
            if (this.ContentGrid != null && this.PopupContentGrid != null && this.ContentPresenter != null)
            {
                this.ContentGrid.Children.Remove(this.ContentPresenter);
                this.PopupContentGrid.Children.Add(this.ContentPresenter);
                //this.PopupContentGrid.Measure(this.infiniteSize);
            }
        }

        private void SetContentToGrid() 
        {
            if (this.ContentGrid != null && this.PopupContentGrid != null && this.ContentPresenter != null)
            {
                this.PopupContentGrid.Children.Remove(this.ContentPresenter);
                this.ContentGrid.Children.Add(this.ContentPresenter);
            }
        }

        private bool IsValidIndex(int index) 
        {
            if (this.OutlookBarItems == null) 
            {
                return false;
            }
            return index >= 0 && index < this.OutlookBarItems.Count;
        }
        
        private void SelectItem(OutlookBarItem item)
        {
            OutlookBarItem previousItem = this.GetValue(OutlookBar.SelectedItemProperty) as OutlookBarItem;
            if (previousItem != null && previousItem == item)
            {
                return;
            }
            if (previousItem != item)
            {
                if (previousItem != null)
                {
                    previousItem.SetValue(OutlookBarItem.IsSelectedPropertyKey, false);
                    this.ClearValue(OutlookBar.SelectedItemProperty);
                }
                this.SetValue(OutlookBar.SelectedItemProperty, item);
                int index = this.ItemsStackPanel.Children.IndexOf(item);
                this.SetValue(OutlookBar.SelectedIndexProperty, index);
                item.SetValue(OutlookBarItem.IsSelectedPropertyKey, true);
            }
        }
        
        internal void SelectItemByIndex(int index)
        {
            if (this.IsValidIndex(index))
            {
                OutlookBarItem selectedItem = this.SelectedItem;
                OutlookBarItem newItem = this.OutlookBarItems[index] as OutlookBarItem;
                if (newItem != selectedItem && newItem != null)
                {
                    this.SelectItem(newItem);
                }
            }
        }

        private void CloseDropDown() 
        {
            if (this.IsDropDownOpen)
            {
                this.IsDropDownOpen = false;
            }
        }

        #endregion

        #region Event Handlers

        private void OutlookBarItemClickedEventRaised(object sender, System.EventArgs e) 
        {
            if (sender is OutlookBarItem) 
            {
                this.SelectItem(sender as OutlookBarItem);
            }
        }

        private void PopupDissmissEventRaised(object sender, System.EventArgs e) 
        {
            this.CloseDropDown();
        }

        private void ExpandCommandExecuted(object sender, System.Windows.Input.ExecutedRoutedEventArgs e) 
        {
            this.IsExpanded = !this.IsExpanded;
        }

        private void ThumbDragDelta(object sender, System.Windows.Controls.Primitives.DragDeltaEventArgs e)
        {
            System.Windows.Controls.Primitives.Thumb thumb = (System.Windows.Controls.Primitives.Thumb)sender;
            if (this.RootGrid == null) 
            {
                thumb.CancelDrag();
                e.Handled = true;
                return;
            }
            double width = this.Direction == Controls.OutlookBar.Direction.Right ? this.RootGrid.Width + e.HorizontalChange : this.RootGrid.Width - e.HorizontalChange;
            
            if ((!(System.Double.IsPositiveInfinity(this.MaxWidth) || System.Double.IsNaN(this.MaxWidth)) && width >= this.MaxWidth))
            {
                thumb.CancelDrag();
                e.Handled = true;
                return;
            }
            if (width <= 75 && this.IsExpanded)
            {
                this.SetValue(OutlookBar.IsExpandedProperty, false);
                this.RootGrid.SetValue(System.Windows.Controls.Grid.WidthProperty, 40.0D);
                thumb.CancelDrag();
                e.Handled = true;
                return;
            }
            else if (width > 95)
            {
                if (!this.IsExpanded)
                {
                    this.SetValue(OutlookBar.IsExpandedProperty, true);
                }
                this.RootGrid.SetValue(System.Windows.Controls.Grid.WidthProperty, width);
                if (this.LoweredItemsPanel.Children.Count > 0 && width - this.LoweredItemsPanel.ActualWidth <= 28)
                {
                    OutlookBarItem item = this.LoweredItemsPanel.Children[this.LoweredItemsPanel.Children.Count - 1] as OutlookBarItem;
                    this.LoweredItemsPanel.Children.Remove(item);
                    this.CollapsedItems.Add(item);
                }
                else if (width - this.LoweredItemsPanel.ActualWidth >= 60 && this.CollapsedItems.Count > 0)
                {
                    OutlookBarItem item = this.CollapsedItems[this.CollapsedItems.Count - 1];
                    this.CollapsedItems.Remove(item);
                    int index = this.LoweredItemsPanel.Children.Count > 0 ? this.LoweredItemsPanel.Children.Count - 1 : 0;
                    this.LoweredItemsPanel.Children.Insert(index, item);
                }
                this.PreviousWidth = width;
            }
        }

        private void HorizontalThumbDragDelta(object sender, System.Windows.Controls.Primitives.DragDeltaEventArgs e)
        {
            if (e.VerticalChange >= 25)
            {
                if (this.ItemsStackPanel != null && this.ItemsStackPanel.Children.Count > 0)
                {
                    OutlookBarItem item = this.ItemsStackPanel.Children[this.ItemsStackPanel.Children.Count - 1] as OutlookBarItem;
                    if (item == null)
                    {
                        return;
                    }
                    this.ItemsStackPanel.Children.Remove(item);
                    item.Size = Ribbon2010.RibbonControlSize.Small;
                    this.LoweredItemsPanel.Children.Insert(0, item);
                }
            }
            else if (e.VerticalChange <= -25)
            {
                if (this.ItemsStackPanel != null && this.LoweredItemsPanel != null && this.LoweredItemsPanel.Children.Count > 0)
                {
                    OutlookBarItem item = this.LoweredItemsPanel.Children[0] as OutlookBarItem;
                    if (item == null)
                    {
                        return;
                    }
                    this.LoweredItemsPanel.Children.Remove(item);
                   
                    item.Size = Ribbon2010.RibbonControlSize.Large;
                    this.ItemsStackPanel.Children.Add(item);
                    if (this.CollapsedItems.Count > 0)
                    {
                        item = this.CollapsedItems[0];
                        this.CollapsedItems.Remove(item);
                        this.LoweredItemsPanel.Children.Add(item);
                    }
                }
            }

        }
        #endregion

        #region Fields

        private static System.EventArgs privateArgs = new System.EventArgs();
        private System.Windows.Controls.Primitives.Thumb LeftThumb, RightThumb, HorizontalThumb;
        private System.Windows.Controls.Grid RootGrid, ItemsGrid;
        private const string RightThumbName = "PART_RightOutlookBarThumb";
        private const string LeftThumbName = "PART_LeftOutlookBarThumb";
        private const string PART_OutlookBarRoot = "PART_OutlookBarRoot";
        private const string HorizontalThumbName = "PART_OutlookBarHorizontalThumb";
        private const string ItemsGridName = "OutlookBarItemsGrid";
        private const string PART_OutlookBarLoweredItemsPanelName = "PART_OutlookBarLoweredItemsPanel";
        private const string PART_ExpandButtonName = "PART_ExpandButton";
        private const string PART_OutlookBar_PopupName = "PART_OutlookBar_Popup";
        private const string PART_OutlookBarContentGridName = "PART_OutlookBarContentGrid";
        private const string PART_OutlookBar_PopupContentGridName = "PART_OutlookBar_PopupContentGrid";
        private const string PART_OutlookBarContentPresenterName = "PART_OutlookBarContentPresenter";
        private System.Windows.Controls.Primitives.Popup Popup;
        private System.Windows.Controls.StackPanel ItemsStackPanel, LoweredItemsPanel;
        private System.Windows.Controls.Grid ContentGrid, PopupContentGrid;
        private System.Windows.Controls.ContentPresenter ContentPresenter;
        //private System.Windows.Size infiniteSize = new System.Windows.Size(System.Double.PositiveInfinity, 700);
        System.Collections.Generic.List<OutlookBarItem> CollapsedItems = new System.Collections.Generic.List<OutlookBarItem>();
        #endregion

        #region Events

        internal static event System.EventHandler OutlookBarItemClicked;

        #endregion

        #region Commands

        private static System.Windows.Input.RoutedUICommand expandCommand = new System.Windows.Input.RoutedUICommand("Expand", "ExpandCommand", typeof(OutlookBar));

        public static System.Windows.Input.RoutedUICommand ExpandCommand 
        {
            get 
            {
                return OutlookBar.expandCommand;
            }
        }
        #endregion
    }
}
