﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Masterpiece.Controls.Primitives
{
    [System.ComponentModel.DefaultEvent("Click")]
    [System.Windows.Localizability(System.Windows.LocalizationCategory.Button)]
    public abstract class ButtonBase : System.Windows.Controls.ContentControl, System.Windows.Input.ICommandSource
    {
        #region Constructors

        static ButtonBase() 
        {
            System.Type ownerType=typeof(ButtonBase);
            System.Windows.EventManager.RegisterClassHandler(ownerType, System.Windows.Input.AccessKeyManager.AccessKeyPressedEvent,
                new System.Windows.Input.AccessKeyPressedEventHandler(OnAccessKeyPressed));
            System.Windows.Input.KeyboardNavigation.AcceptsReturnProperty.OverrideMetadata
                (
                ownerType, new System.Windows.FrameworkPropertyMetadata(false)
                );
            System.Windows.Input.InputMethod.IsInputMethodEnabledProperty.OverrideMetadata(ownerType, new System.Windows.FrameworkPropertyMetadata(false, System.Windows.FrameworkPropertyMetadataOptions.Inherits));
            ButtonBase.SnapsToDevicePixelsProperty.OverrideMetadata
                (
                ownerType, new System.Windows.FrameworkPropertyMetadata(true, new System.Windows.PropertyChangedCallback(OnSnapsToDevicePixelsPropertyChanged), new System.Windows.CoerceValueCallback(IsValidBooleanValue))
                );
        }

        public ButtonBase()
            : base()
        {
        }

        #endregion

        #region Properties and Depenency Properties

        /// <summary>
        ///  Gets a value whether button is highlighted. This is a dependency property.
        /// </summary>
        internal bool IsHighlighted
        {
            get { return (bool)GetValue(IsHighlightedProperty); }
            set { SetValue(IsHighlightedProperty, value); }
        }

        // Using a DependencyProperty as the backing store for IsHighlighted.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty IsHighlightedProperty =
            System.Windows.DependencyProperty.Register("IsHighlighted", typeof(bool), typeof(ButtonBase), new System.Windows.FrameworkPropertyMetadata(false));
        
        /// <summary>
        /// Gets a value whether button state is activated. This is a dependency property.
        /// </summary>
        [System.ComponentModel.Browsable(false), System.ComponentModel.Category("Appearance"), System.ComponentModel.ReadOnly(true)]
        public bool IsPressed
        {
            get { return (bool)GetValue(IsPressedProperty); }
        }

        // Using a DependencyProperty as the backing store for IsPressed.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyPropertyKey IsPressedPropertyKey =
            System.Windows.DependencyProperty.RegisterReadOnly("IsPressed", typeof(bool), typeof(ButtonBase), new System.Windows.PropertyMetadata(false));

        internal static readonly System.Windows.DependencyProperty IsPressedProperty = IsPressedPropertyKey.DependencyProperty;

        /// <summary>
        ///Gets or sets Command property of button. This is a dependency property.
        /// </summary>
        
        [System.ComponentModel.Bindable(true), 
        System.ComponentModel.Browsable(false), 
        System.ComponentModel.ReadOnly(false)]
        public System.Windows.Input.ICommand Command
        {
            get { return (System.Windows.Input.ICommand)GetValue(CommandProperty); }
            set { SetValue(CommandProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Command.  This enables animation, styling, binding, etc...
        
        public static readonly System.Windows.DependencyProperty CommandProperty =
            System.Windows.DependencyProperty.Register("Command", typeof(System.Windows.Input.ICommand), typeof(ButtonBase), new System.Windows.PropertyMetadata((System.Windows.Input.ICommand)null, new System.Windows.PropertyChangedCallback(OnCommandPropertyChanged)));

        /// <summary>
        /// Gets or sets command target of button. This is a dependency property.
        /// </summary>
        [System.ComponentModel.Bindable(true), System.ComponentModel.Category("Action")]
        public System.Windows.IInputElement CommandTarget
        {
            get { return (System.Windows.IInputElement)GetValue(CommandTargetProperty); }
            set { SetValue(CommandTargetProperty, value); }
        }

        // Using a DependencyProperty as the backing store for CommandTarget.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty CommandTargetProperty =
            System.Windows.DependencyProperty.Register("CommandTarget", typeof(System.Windows.IInputElement), typeof(ButtonBase), new System.Windows.PropertyMetadata((System.Windows.IInputElement)null));


        public System.Windows.CornerRadius CornerRadius
        {
            get { return (System.Windows.CornerRadius)GetValue(CornerRadiusProperty); }
            set { SetValue(CornerRadiusProperty, value); }
        }

        // Using a DependencyProperty as the backing store for CornerRadius.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty CornerRadiusProperty =
            System.Windows.DependencyProperty.Register("CornerRadius", typeof(System.Windows.CornerRadius), typeof(ButtonBase), new System.Windows.PropertyMetadata(null));
        
        /// <summary>
        /// Gets or sets command parameter of button. This is a dependency property.
        /// </summary>
        [System.ComponentModel.Bindable(true), System.ComponentModel.Category("Action")]
        [System.Windows.Localizability(System.Windows.LocalizationCategory.NeverLocalize)]
        public object CommandParameter
        {
            get { return (object)GetValue(CommandParameterProperty); }
            set { SetValue(CommandParameterProperty, value); }
        }

        // Using a DependencyProperty as the backing store for CommandParameter.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty CommandParameterProperty =
            System.Windows.DependencyProperty.Register("CommandParameter", typeof(object), typeof(ButtonBase), new System.Windows.PropertyMetadata((object)null));

        public static readonly System.Windows.DependencyProperty ClickModeProperty =
               System.Windows.DependencyProperty.Register(
                       "ClickMode",
                       typeof(System.Windows.Controls.ClickMode),
                       typeof(ButtonBase),
                       new System.Windows.FrameworkPropertyMetadata(System.Windows.Controls.ClickMode.Release),
                       new System.Windows.ValidateValueCallback(IsValidClickMode));

        [System.ComponentModel.Bindable(true), System.ComponentModel.Category("Behavior")]
        public System.Windows.Controls.ClickMode ClickMode
        {
            get
            {
                return (System.Windows.Controls.ClickMode)GetValue(ClickModeProperty);
            }
            set
            {
                SetValue(ClickModeProperty, value);
            }
        }

        #endregion

        #region Depenency Property Changed Callbacks

        /// <summary>
        /// Invokes whn command property changes.
        /// </summary>
        /// <param name="o"></param>
        /// <param name="e"></param>
        private static void OnCommandPropertyChanged(System.Windows.DependencyObject o, System.Windows.DependencyPropertyChangedEventArgs e)
        {
            ButtonBase btn = o as ButtonBase;
            btn.SetValue(ButtonBase.CommandTargetProperty, btn);
            btn.HookUpCommand(e.OldValue as System.Windows.Input.ICommand, e.NewValue as System.Windows.Input.ICommand);
        }

        /// <summary>
        /// Called when ButtonBase.SnapsToDevicePixelsProperty changes value
        /// </summary>
        /// <param name="o"></param>
        /// <param name="e"></param>
        private static void OnSnapsToDevicePixelsPropertyChanged(System.Windows.DependencyObject o, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
        }

        /// <summary>
        /// Called before calling OnSnapsToDevicePixelsPropertyChanged method
        /// </summary>
        /// <param name="o"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        private static object IsValidBooleanValue(System.Windows.DependencyObject o, object value) 
        {
            bool? val = (bool?)value;
            if (!val.HasValue) 
            {
                return true;
            }
            return value is bool ? value : true;
        }
        #endregion

        #region Overrides

        /// <summary>
        /// Invokes when mouse left button down.
        /// </summary>
        /// <param name="e"></param>
        protected override void OnMouseLeftButtonDown(System.Windows.Input.MouseButtonEventArgs e)
        {
            
            if (this.ClickMode != System.Windows.Controls.ClickMode.Hover) 
            {
                e.Handled = true;
              //  this.Focus();
                if (e.ButtonState == System.Windows.Input.MouseButtonState.Pressed) 
                {
                    this.CaptureMouse();
                    if (this.IsMouseCaptured) 
                    {
                        if (e.ButtonState == System.Windows.Input.MouseButtonState.Pressed)
                        {
                            if (!this.IsPressed)
                            {
                                this.SetIsPressed(true);
                            }
                        }
                        else 
                        {
                            this.ReleaseMouseCapture();
                        }
                    }
                }

                if (this.ClickMode == System.Windows.Controls.ClickMode.Press) 
                {
                    bool exceptionThrown = true;
                    try
                    {
                       this.OnClick();
                        
                        exceptionThrown = false;
                    }
                    finally 
                    {
                        if (exceptionThrown) 
                        {
                            this.SetIsPressed(false);
                            this.ReleaseMouseCapture();
                        }
                    }
                }
            }
            base.OnMouseLeftButtonDown(e);
        }

        /// <summary>
        /// Invokes when mouse left button up.
        /// </summary>
        /// <param name="e"></param>
        protected override void OnMouseLeftButtonUp(System.Windows.Input.MouseButtonEventArgs e)
        {
            if (this.ClickMode != System.Windows.Controls.ClickMode.Hover) 
            {
                e.Handled = true;
                bool shouldClick = this.IsPressed && this.ClickMode == System.Windows.Controls.ClickMode.Release;
                if (this.IsMouseCaptured) 
                {
                    this.ReleaseMouseCapture();
                }
                if (shouldClick)
                {
                    this.OnClick();
                }

              //  this.SetIsPressed(false);
            }

            base.OnMouseLeftButtonUp(e);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="e"></param>
        protected override void OnMouseMove(System.Windows.Input.MouseEventArgs e)
        {
            base.OnMouseMove(e);

            if (this.ClickMode != System.Windows.Controls.ClickMode.Hover && (this.IsMouseCaptured && System.Windows.Input.Mouse.PrimaryDevice.LeftButton == System.Windows.Input.MouseButtonState.Pressed)) 
            {
                this.UpdateIsPressed();
                e.Handled = true;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="e"></param>
        protected override void OnLostMouseCapture(System.Windows.Input.MouseEventArgs e)
        {
            base.OnLostMouseCapture(e);
            if (e.OriginalSource == this && this.ClickMode != System.Windows.Controls.ClickMode.Hover) 
            {
                if (this.IsKeyboardFocused && !this.IsInMainFocus) 
                {
                    System.Windows.Input.Keyboard.Focus(null);
                }
                this.SetIsPressed(false);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="e"></param>
        protected override void OnMouseEnter(System.Windows.Input.MouseEventArgs e)
        {
            base.OnMouseEnter(e);
            if (this.HandleIsMouseOverChanged()) 
            {
                e.Handled = true;
            }
        }
        
        /// <summary>
        /// 
        /// </summary>
        /// <param name="e"></param>
        protected override void OnMouseLeave(System.Windows.Input.MouseEventArgs e)
        {
            base.OnMouseLeave(e);
            if (this.HandleIsMouseOverChanged()) 
            {
                e.Handled = true;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="e"></param>
        protected override void OnKeyDown(System.Windows.Input.KeyEventArgs e)
        {
            base.OnKeyDown(e);
            if (this.ClickMode == System.Windows.Controls.ClickMode.Hover) 
            {
                return;
            }
            if (e.Key == System.Windows.Input.Key.Space) 
            {
                if ((System.Windows.Input.Keyboard.Modifiers & (System.Windows.Input.ModifierKeys.Control | System.Windows.Input.ModifierKeys.Alt)) != System.Windows.Input.ModifierKeys.Alt) 
                {
                    if ((!this.IsMouseCaptured) && e.OriginalSource == this) 
                    {
                        this.SetIsPressed(true);
                        this.CaptureMouse();
                        if (this.ClickMode == System.Windows.Controls.ClickMode.Press) 
                        {
                            this.OnClick();
                        }

                        e.Handled = true;
                    }
                }
            }
            else if (e.Key == System.Windows.Input.Key.Enter && (bool)this.GetValue(System.Windows.Input.KeyboardNavigation.AcceptsReturnProperty))
            {
                if (e.OriginalSource == this)
                {
                    this.SetIsPressed(false);
                    if (this.IsMouseCaptured)
                    {
                        this.ReleaseMouseCapture();
                    }

                    this.OnClick();
                    e.Handled = true;
                }
            }
            else 
            {
                this.SetIsPressed(false);
                if (this.IsMouseCaptured) 
                {
                    this.ReleaseMouseCapture();
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="e"></param>
        protected override void OnKeyUp(System.Windows.Input.KeyEventArgs e)
        {
            base.OnKeyUp(e);

            if (this.ClickMode == System.Windows.Controls.ClickMode.Hover)
            {
                // Ignore when in hover-click mode.
                return;
            }

            if ((e.Key == System.Windows.Input.Key.Space))
            {
                // Alt+Space should bring up system menu, we shouldn't handle it.
                if ((System.Windows.Input.Keyboard.Modifiers & (System.Windows.Input.ModifierKeys.Control | System.Windows.Input.ModifierKeys.Alt)) != System.Windows.Input.ModifierKeys.Alt)
                {
                    if (this.GetMouseLeftButtonReleased())
                    {
                        bool shouldClick = this.IsPressed && this.ClickMode == System.Windows.Controls.ClickMode.Release;

                        // Release mouse capture if left mouse button is not pressed
                        if (IsMouseCaptured)
                        {
                            // OnLostMouseCapture set IsPressed to false
                            this.ReleaseMouseCapture();
                        }

                        if (shouldClick)
                            this.OnClick();
                    }
                    else
                    {
                        // IsPressed state is updated only if mouse is captured (bugfix 919349)
                        if (this.IsMouseCaptured)
                            this.UpdateIsPressed();
                    }

                    e.Handled = true;
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="e"></param>
        protected override void OnLostKeyboardFocus(System.Windows.Input.KeyboardFocusChangedEventArgs e)
        {
            base.OnLostKeyboardFocus(e);

            if (ClickMode == System.Windows.Controls.ClickMode.Hover)
            {
                // Ignore when in hover-click mode.
                return;
            }

            if (e.OriginalSource == this)
            {
                if (this.IsPressed)
                {
                    this.SetIsPressed(false);
                }

                if (this.IsMouseCaptured)
                    this.ReleaseMouseCapture();

            }
        }
        
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [System.Security.SecurityCritical, System.Security.SecurityTreatAsSafe]
        private bool GetMouseLeftButtonReleased()
        {
            return System.Windows.Input.InputManager.Current.PrimaryMouseDevice.LeftButton == System.Windows.Input.MouseButtonState.Released;
        }

        /// <summary>
        /// 
        /// </summary>
        protected virtual void OnClick()
        {
            Masterpiece.RoutedEventArgs args = new RoutedEventArgs(ButtonBase.ClickEvent, this);
            
                    bool exceptionThrown = true;
                    try
                    {
                        this.RaiseEvent(args);

                        if (this.Command != null)
                        {
                            System.Windows.Input.RoutedCommand commad = this.Command as System.Windows.Input.RoutedCommand;

                            if (commad != null)
                            {
                                commad.Execute(this.CommandParameter, this.CommandTarget);
                            }
                            else
                            {
                                (this.Command as System.Windows.Input.ICommand).Execute(this.CommandParameter);
                            }
                        }
                        exceptionThrown = false;
                    }
                    finally 
                    {
                        if (exceptionThrown) 
                        {
                            throw new System.InvalidOperationException("Cannot raise Click event.");
                        }
                   }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sizeInfo"></param>
        protected override void OnRenderSizeChanged(System.Windows.SizeChangedInfo sizeInfo)
        {
            base.OnRenderSizeChanged(sizeInfo);
            if (this.IsMouseCaptured && (System.Windows.Input.Mouse.PrimaryDevice.LeftButton == System.Windows.Input.MouseButtonState.Pressed)) 
            {
                this.UpdateIsPressed();
            }
        }

        #endregion

        #region Private Utilities

        //internal void SetManuallyMouseOverState()
        //{
        //    Masterpiece.Controls.Ribbon2010.RibbonChrome chrome = this.GetTemplateChild("PART_RibbonChrome") as Masterpiece.Controls.Ribbon2010.RibbonChrome;
        //    if (chrome != null)
        //    {
        //        System.Windows.Data.Binding binding = System.Windows.Data.BindingOperations.GetBinding(chrome, Controls.Ribbon2010.RibbonChrome.RenderMouseOverProperty);
        //        System.Windows.Data.BindingExpression ex = chrome.GetBindingExpression(Controls.Ribbon2010.RibbonChrome.RenderMouseOverProperty);
        //        if (ex != null) 
        //        {
                
        //        }
        //        if (binding != null) 
        //        {
                
                
        //        }
        //        chrome.RenderMouseOver = true;
        //    }
        //}

        //internal void UnSetManuallyMouseOverState()
        //{
        //    Masterpiece.Controls.Ribbon2010.RibbonChrome chrome = this.GetTemplateChild("PART_RibbonChrome") as Masterpiece.Controls.Ribbon2010.RibbonChrome;
        //    if (chrome != null)
        //    {
        //        System.Windows.Data.BindingOperations.SetBinding
        //            (
        //                chrome,
        //                Controls.Ribbon2010.RibbonChrome.RenderMouseOverProperty,
        //                new System.Windows.Data.Binding("IsMouseOver") { Source = this }
        //            );
        //    }
        //}

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        private bool HandleIsMouseOverChanged()
        {
            if (this.ClickMode == System.Windows.Controls.ClickMode.Hover)
            {
                if (this.IsMouseOver)
                {
                    // Hovering over the button will click in the OnHover click mode
                    this.SetIsPressed(true);
                    this.OnClick();
                }
                else
                {
                    this.SetIsPressed(false);
                }

                return true;
            }

            return false;
        }

        /// <summary>
        /// 
        /// </summary>
        private void UpdateIsPressed()
        {
            System.Windows.Point pos = System.Windows.Input.Mouse.PrimaryDevice.GetPosition(this);

            if ((pos.X >= 0) && (pos.X <= this.ActualWidth) && (pos.Y >= 0) && (pos.Y <= this.ActualHeight))
            {
                if (!this.IsPressed)
                {
                    this.SetIsPressed(true);
                }
            }
            else if (this.IsPressed)
            {
                this.SetIsPressed(false);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public bool IsInMainFocus 
        {
            get 
            {
                System.Windows.Media.Visual visual = System.Windows.Input.FocusManager.GetFocusScope(this) as System.Windows.Media.Visual;
                
                return visual == null || System.Windows.Media.VisualTreeHelper.GetParent(visual)==null;
            } 
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="pressed"></param>
        private void SetIsPressed(bool pressed)
        {
            if (pressed)
            {
                this.SetValue(IsPressedPropertyKey, pressed);
            }
            else
            {
                this.ClearValue(IsPressedPropertyKey);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        internal void AutomationButtonBaseClick() 
        {
            this.OnClick();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        private static bool IsValidClickMode(object value) 
        {
            System.Windows.Controls.ClickMode clickMode = (System.Windows.Controls.ClickMode)value;
            return clickMode == System.Windows.Controls.ClickMode.Press 
                || clickMode == System.Windows.Controls.ClickMode.Release 
                || clickMode == System.Windows.Controls.ClickMode.Hover;
        }

        /// <summary>
        /// Established command of button.
        /// </summary>
        /// <param name="oldCommand"></param>
        /// <param name="newCommand"></param>
        private void HookUpCommand(System.Windows.Input.ICommand oldCommand, System.Windows.Input.ICommand newCommand)
        {
            if (oldCommand != null)
            {
                this.RemoveCommand(oldCommand, newCommand);
            }

            this.AddCommand(oldCommand, newCommand);
        }

        /// <summary>
        /// Removes old command.
        /// </summary>
        /// <param name="oldCommand"></param>
        /// <param name="newCommand"></param>
        private void RemoveCommand(System.Windows.Input.ICommand oldCommand, System.Windows.Input.ICommand newCommand)
        {
            System.EventHandler handler = this.CanExecuteChanged;
            oldCommand.CanExecuteChanged -= handler;
        }

        /// <summary>
        /// Sets new command.
        /// </summary>
        /// <param name="oldCommand"></param>
        /// <param name="newCommand"></param>
        private void AddCommand(System.Windows.Input.ICommand oldCommand, System.Windows.Input.ICommand newCommand)
        {
            System.EventHandler handler = new EventHandler(this.CanExecuteChanged);
            canExecuteChangedHandler = handler;
            if (newCommand != null)
            {
                newCommand.CanExecuteChanged += canExecuteChangedHandler;
            }
        }

        /// <summary>
        /// Checks whether can execute or not.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CanExecuteChanged(object sender, System.EventArgs e)
        {
            if (this.Command != null)
            {
                System.Windows.Input.RoutedCommand command = this.Command as System.Windows.Input.RoutedCommand;
                //If Routed command
                if (command != null)
                {
                    if (command.CanExecute(this.CommandParameter, this.CommandTarget))
                    {
                        this.IsEnabled = true;
                    }
                    else
                    {
                        this.IsEnabled = false;
                    }
                }
                // If not a routed command
                else
                {
                    if (this.Command.CanExecute(this.CommandParameter))
                    {
                        this.IsEnabled = true;
                    }
                    else
                    {
                        this.IsEnabled = false;
                    }
                }
            }
        }

        #endregion
        
        #region Event Handlers

        private static void OnAccessKeyPressed(object sender, System.Windows.Input.AccessKeyPressedEventArgs e) 
        {
        
        }

        #endregion

        #region Fields

        private static System.EventHandler canExecuteChangedHandler;
        #endregion

        /// <summary>
        /// Registering new Routed Event for a button.
        /// </summary>
        public static readonly System.Windows.RoutedEvent ClickEvent = System.Windows.EventManager.RegisterRoutedEvent
            (
            "Click",
             System.Windows.RoutingStrategy.Bubble,
             typeof(Masterpiece.RoutedEventHandler),
             typeof(ButtonBase)
            );

        /// <summary>
        /// 
        /// </summary>
        [System.ComponentModel.Category("Behavior")]
        public event RoutedEventHandler Click
        {
            add { this.AddHandler(ButtonBase.ClickEvent, value); }
            remove { this.RemoveHandler(ButtonBase.ClickEvent, value); }
        }

    }
}
