﻿
namespace Masterpiece.Controls.Primitives
{
    [System.ComponentModel.DefaultEvent("DragDelta")]
    [System.Windows.Localizability(System.Windows.LocalizationCategory.NeverLocalize)]
    public class Thumb:System.Windows.Controls.Control
    {
        #region Constructors

        static Thumb() 
        {
            System.Type ownerType=typeof(Thumb);
            Thumb.CursorProperty.OverrideMetadata(ownerType, new System.Windows.FrameworkPropertyMetadata(System.Windows.Input.Cursors.SizeNS));
            Thumb.DefaultStyleKeyProperty.OverrideMetadata(ownerType, new System.Windows.FrameworkPropertyMetadata(ownerType));
        }

        /// <summary>
        /// 
        /// </summary>
        public Thumb() 
        {
           
        }

        #endregion

        #region Properties and Depenency Properties

        /// <summary>
        /// 
        /// </summary>
        [System.ComponentModel.Bindable(true), System.ComponentModel.Browsable(false), System.ComponentModel.Category("Appearance")]
        public bool IsDragging
        {
            get { return (bool)GetValue(IsDraggingProperty); }
        }

        // Using a DependencyProperty as the backing store for IsDragging.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyPropertyKey IsDraggingPropertyKey =
            System.Windows.DependencyProperty.RegisterReadOnly("IsDragging", typeof(bool), typeof(Thumb), new System.Windows.PropertyMetadata(false, new System.Windows.PropertyChangedCallback(OnIsDraggingPropertyChanged)));
        internal static readonly System.Windows.DependencyProperty IsDraggingProperty = IsDraggingPropertyKey.DependencyProperty;
        

        #endregion

        #region Depenency Property Changed Callbacks

        private static void OnIsDraggingPropertyChanged(System.Windows.DependencyObject o, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
        
        }
        #endregion

        #region Overrides

        /// <summary>
        /// 
        /// </summary>
        /// <param name="e"></param>
        protected override void OnMouseLeftButtonDown(System.Windows.Input.MouseButtonEventArgs e)
        {
            if (!this.IsDragging)
            {
                e.Handled = true;
                this.Focus();
                this.CaptureMouse();
                this.SetValue(Thumb.IsDraggingPropertyKey, true);
                this._originThumbPoint = e.GetPosition(this);
                this._previousScreenCoordPosition = this._originScreenCoordPosition = this.PointToScreen(this._originThumbPoint);
                bool exceptionThrown = true;
                DragStartedEventArgs args = new DragStartedEventArgs(this._originThumbPoint.X, this._originThumbPoint.Y);
                try
                {
                    this.RaiseEvent(args);
                    exceptionThrown = false;
                }
                finally
                {
                    if (exceptionThrown)
                    {
                        this.CancelDrag();
                    }
                }
            }
            else 
            {
                
            }

            base.OnMouseLeftButtonDown(e);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="e"></param>
        protected override void OnMouseMove(System.Windows.Input.MouseEventArgs e)
        {
            base.OnMouseMove(e);
            if (this.IsDragging) 
            {
                if (e.MouseDevice.LeftButton == System.Windows.Input.MouseButtonState.Pressed) 
                {
                    System.Windows.Point thumbCoordPosition = e.GetPosition(this);
                    System.Windows.Point screenCoordPosition = this.ScreenToLocal(thumbCoordPosition);
                    if (screenCoordPosition != this._previousScreenCoordPosition)
                    {
                        this._previousScreenCoordPosition = screenCoordPosition;
                        e.Handled = true;
                        this.RaiseEvent(new DragDeltaEventArgs(thumbCoordPosition.X - this._originThumbPoint.X,
                                                          thumbCoordPosition.Y - this._originThumbPoint.Y));
                    }
                }
                else
                {
                    if (e.MouseDevice.Captured == this)
                        this.ReleaseMouseCapture();
                    this.ClearValue(IsDraggingPropertyKey);
                    _originThumbPoint.X = 0;
                    _originThumbPoint.Y = 0;
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="e"></param>
        protected override void OnMouseLeftButtonUp(System.Windows.Input.MouseButtonEventArgs e)
        {
            if (this.IsDragging && this.IsMouseCaptured) 
            {
                e.Handled = true;
                this.ClearValue(Thumb.IsDraggingPropertyKey);
                this.ReleaseMouseCapture();
                System.Windows.Point pt = this.ScreenToLocal(e.MouseDevice.GetPosition(this));
                this.RaiseEvent(new DragCompletedEventArgs(pt.X-this._originScreenCoordPosition.X, pt.Y-this._originScreenCoordPosition.Y, false));
            }
            base.OnMouseLeftButtonUp(e);
        }
        private static void OnLostMouseCapture(object sender, System.Windows.Input.MouseEventArgs e)
        {
            Thumb thumb = (Thumb)sender;

            if (System.Windows.Input.Mouse.Captured != thumb)
            {
                thumb.CancelDrag();
            }
        }

        #endregion

        #region Private Utilities
        private System.Windows.Point ScreenToLocal(System.Windows.Point point)
        {
            System.Windows.Interop.HwndSource source = (System.Windows.Interop.HwndSource)System.Windows.PresentationSource.FromVisual(this);
            System.IntPtr handle = source.Handle;
            Masterpiece.Common.NativeMethods.RECT rect;
            Masterpiece.Common.NativeMethods.GetWindowRect(handle, out rect);
            System.Windows.Media.Matrix matrix = System.Windows.PresentationSource.FromVisual(this).CompositionTarget.TransformFromDevice;
            point.Offset((double)(-1 * rect.Left), (double)(-1 * rect.Top));
            point.X *= matrix.M11;
            point.Y *= matrix.M22;
            return point;
        }
        #endregion

        #region Event Handlers
       
        #endregion

        #region PublicMethods

        public void CancelDrag() 
        {
            if (this.IsDragging) 
            {
                if (this.IsMouseCaptured) 
                {
                    this.ReleaseMouseCapture();
                }
            }
            this.SetValue(Thumb.IsDraggingPropertyKey, false);
            DragCompletedEventArgs deltaArgs = new DragCompletedEventArgs(this._previousScreenCoordPosition.X-this._originScreenCoordPosition.X, this._previousScreenCoordPosition.Y-this._originScreenCoordPosition.Y, true);
            this.RaiseEvent(deltaArgs);
        }

        #endregion

        #region Fields

        
        private System.Windows.Point _originScreenCoordPosition;
        private System.Windows.Point _previousScreenCoordPosition;
        private System.Windows.Point _originThumbPoint;        
        private System.Windows.Window MainWindow;
        #endregion

        #region Events

        /// <summary>
        /// 
        /// </summary>
        [System.ComponentModel.Category("Behavior")]
        public event DragStartedEventHandler DragStarted 
        {
            add { this.AddHandler(Thumb.DragStartedEvent, value); }
            remove { this.RemoveHandler(Thumb.DragStartedEvent, value); }
        }
        
        /// <summary>
        /// 
        /// </summary>
        public static readonly System.Windows.RoutedEvent DragStartedEvent = System.Windows.EventManager.RegisterRoutedEvent
            (
            "DragStarted",
             System.Windows.RoutingStrategy.Bubble,
             typeof(DragStartedEventHandler),
             typeof(Thumb)
            );
        
        /// <summary>
        /// 
        /// </summary>
        [System.ComponentModel.Category("Behavior")]
        public event DragDeltaEventHandler DragDelta 
        {
            add { this.AddHandler(Thumb.DragDeltaEvent, value); }
            remove { this.RemoveHandler(Thumb.DragDeltaEvent, value); }
        }
        
        
        /// <summary>
        /// 
        /// </summary>
        public static readonly System.Windows.RoutedEvent DragDeltaEvent = System.Windows.EventManager.RegisterRoutedEvent
           (
           "DragDelta",
            System.Windows.RoutingStrategy.Bubble,
            typeof(DragDeltaEventHandler),
            typeof(Thumb)
           );

        /// <summary>
        /// 
        /// </summary>
        [System.ComponentModel.Category("Behavior")]
        public event DragCompletedEventHandler DragCompeted 
        {
            add { this.AddHandler(Thumb.DragCompletedEvent, value); }
            remove { this.RemoveHandler(Thumb.DragCompletedEvent, value); }
        }
        
        /// <summary>
        /// 
        /// </summary>
        public static readonly System.Windows.RoutedEvent DragCompletedEvent = System.Windows.EventManager.RegisterRoutedEvent
           (
           "DragCompleted",
            System.Windows.RoutingStrategy.Bubble,
            typeof(DragCompletedEventHandler),
            typeof(Thumb)
           );
        #endregion


    }
}
