﻿

namespace Masterpiece.Controls.Primitives
{
    public class ToggleButton : ButtonBase
    {
         #region Constructors

        static ToggleButton() 
        {
            System.Type ownerType=typeof(ToggleButton);
            DefaultStyleKeyProperty.OverrideMetadata(ownerType,
            new System.Windows.FrameworkPropertyMetadata(ownerType));
        }
        public ToggleButton():base() 
        {
        }
        #endregion
        
        #region Properties and Depenency Properties
        [System.ComponentModel.Category("Appearance")]
        [System.ComponentModel.TypeConverter(typeof(System.Windows.NullableBoolConverter))]
        [System.Windows.Localizability(System.Windows.LocalizationCategory.None, Readability = System.Windows.Readability.Unreadable)]
        [System.ComponentModel.Bindable(true), System.ComponentModel.Browsable(false), System.ComponentModel.ReadOnly(false)]
        public bool? IsChecked
        {
            get 
            {
                object value = this.GetValue(IsCheckedProperty);
                if (value == null)
                {
                    return new System.Nullable<bool>();
                }
                else 
                {
                    return new System.Nullable<bool>((bool)value);
                }
            }
            set { SetValue(IsCheckedProperty, value.HasValue ? (bool?)value.Value : null); }
        }

        // Using a DependencyProperty as the backing store for IsChecked.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty IsCheckedProperty =
            System.Windows.DependencyProperty.Register("IsChecked", typeof(bool?), typeof(ToggleButton), new System.Windows.FrameworkPropertyMetadata
                (
                false, 
                System.Windows.FrameworkPropertyMetadataOptions.BindsTwoWayByDefault 
                | System.Windows.FrameworkPropertyMetadataOptions.Journal, 
                new System.Windows.PropertyChangedCallback(OnIsCheckedPropertyChanged)));
        
        #endregion

        #region Depenency Property Changed Callbacks
        
        private static void OnIsCheckedPropertyChanged(System.Windows.DependencyObject o, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
            ToggleButton btn = (ToggleButton)o;
            bool? newValue = (bool?)e.NewValue;
            bool? oldValue = (bool?)e.OldValue;
            if (newValue == true) 
            {
                btn.OnChecked(new System.Windows.RoutedEventArgs(ToggleButton.CheckedEvent));            
            }
            else if (newValue == false) 
            {
                btn.OnUnchecked(new System.Windows.RoutedEventArgs(ToggleButton.UnCheckedEvent));
            }

        }

        #endregion

        #region Overrides

        protected virtual void OnChecked(System.Windows.RoutedEventArgs e) 
        {
            this.RaiseEvent(e);
        }

        protected virtual void OnUnchecked(System.Windows.RoutedEventArgs e) 
        {
            this.RaiseEvent(e);
        }

        protected override void OnClick()
        {
            this.OnToggle();
            if (!(this.TemplatedParent is Ribbon2010.RibbonSplitButton && this.TemplatedParent != null))
            {
                base.OnClick();
            }
        }
        #endregion

        #region Private Utilities
        
        private void OnToggle() 
        {
            if (this.IsChecked == true)
            {
                this.IsChecked = (bool?)false;
            }
            else 
            {
                this.IsChecked = this.IsChecked.HasValue;
            }
        }
        
        #endregion

        #region Event Handlers
        
        [System.ComponentModel.Category("Behavior")]
        public event System.Windows.RoutedEventHandler Checked 
        {
            add { this.AddHandler(ToggleButton.CheckedEvent, value); }
            remove { this.RemoveHandler(ToggleButton.CheckedEvent, value); }
        }
        
        [System.ComponentModel.Category("Behavior")]
        public event System.Windows.RoutedEventHandler UnChecked
        {
            add { this.AddHandler(ToggleButton.UnCheckedEvent, value); }
            remove { this.RemoveHandler(ToggleButton.UnCheckedEvent, value); }
        }

        public static readonly System.Windows.RoutedEvent CheckedEvent = System.Windows.EventManager.RegisterRoutedEvent
            (
            "Checked", 
            System.Windows.RoutingStrategy.Bubble, 
            typeof(System.Windows.RoutedEventHandler), 
            typeof(ToggleButton)
            );
        public static readonly System.Windows.RoutedEvent UnCheckedEvent = System.Windows.EventManager.RegisterRoutedEvent
            (
            "UnChecked",
            System.Windows.RoutingStrategy.Bubble,
            typeof(System.Windows.RoutedEventHandler),
            typeof(ToggleButton)
            );
        #endregion

        #region Fields
        #endregion
    }
}
