﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Masterpiece.Controls.Ribbon2010
{
    [System.Windows.TemplatePart(Name = PART_Root, Type = typeof(RibbonToggleButton))]
    [System.Windows.Markup.ContentProperty("MenuItems")]
    public sealed class RibbonApplicationMenu : System.Windows.Controls.ContentControl, Interfaces.IRibbonControl
    {
        #region Constructors
        
        static RibbonApplicationMenu() 
        {
            DefaultStyleKeyProperty.OverrideMetadata
                (
                typeof(RibbonApplicationMenu),
                new System.Windows.FrameworkPropertyMetadata(typeof(RibbonApplicationMenu))
                );
        }

        public RibbonApplicationMenu() 
        {
            this.MenuItems = new System.Collections.ObjectModel.ObservableCollection<ApplicationMenuItem>();
            this.Border = System.Windows.Application.LoadComponent
                (
                new System.Uri("/Masterpiece;component/Themes/Ribbon2010/RibbonApplicationMenuContent.xaml", System.UriKind.Relative)
                ) as System.Windows.Controls.Border;

            this.ItemsOwner = ((this.Border.Child  as System.Windows.Controls.Grid).Children[0] as System.Windows.Controls.Border).Child as System.Windows.Controls.ItemsControl;
            this.MenuItemContent = ((this.Border.Child as System.Windows.Controls.Grid).Children[1] 
                as System.Windows.Controls.Grid).Children[1] 
                as System.Windows.Controls.ContentPresenter;
        }
        #endregion

        #region Properties and Depenency Properties

        public string Title
        {
            get { return (string)GetValue(TitleProperty); }
            set { SetValue(TitleProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Title.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty TitleProperty =
            System.Windows.DependencyProperty.Register("Title", typeof(string), typeof(RibbonApplicationMenu), new System.Windows.PropertyMetadata(null));


        public System.Collections.ObjectModel.ObservableCollection<ApplicationMenuItem> MenuItems
        {
            get { return (System.Collections.ObjectModel.ObservableCollection<ApplicationMenuItem>)GetValue(MenuItemsProperty); }
            set { SetValue(MenuItemsProperty, value); }
        }

        // Using a DependencyProperty as the backing store for MenuItems.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty MenuItemsProperty=
            System.Windows.DependencyProperty.Register("MenuItems", typeof(System.Collections.ObjectModel.ObservableCollection<ApplicationMenuItem>), typeof(RibbonApplicationMenu), new System.Windows.PropertyMetadata((System.Collections.ObjectModel.ObservableCollection<ApplicationMenuItem>)null));

        public ApplicationMenuItem SelectedItem
        {
            get { return (ApplicationMenuItem)GetValue(SelectedItemProperty); }
            set { SetValue(SelectedItemProperty, value); }
        }

        // Using a DependencyProperty as the backing store for SelectedItem.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty SelectedItemProperty =
            System.Windows.DependencyProperty.Register("SelectedItem", typeof(ApplicationMenuItem), typeof(RibbonApplicationMenu));

        public int SelectedIndex
        {
            get { return (int)GetValue(SelectedIndexProperty); }
            set { SetValue(SelectedIndexProperty, value); }
        }

        // Using a DependencyProperty as the backing store for SelectedIndex.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty SelectedIndexProperty =
            System.Windows.DependencyProperty.Register("SelectedIndex", typeof(int), typeof(RibbonApplicationMenu), new System.Windows.UIPropertyMetadata(-1));

        internal RibbonBar RibbonBar
        {
            get { return (RibbonBar)GetValue(RibbonBarProperty); }
            set { SetValue(RibbonBarProperty, value); }
        }

        // Using a DependencyProperty as the backing store for RibbonBar.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty RibbonBarProperty =
            System.Windows.DependencyProperty.Register("RibbonBar", typeof(RibbonBar), typeof(RibbonApplicationMenu), new System.Windows.PropertyMetadata(null, OnRibbonBarPropertyChanged));

        internal object WindowContent
        {
            get { return (object)GetValue(WindowContentProperty); }
            set { SetValue(WindowContentProperty, value); }
        }

        // Using a DependencyProperty as the backing store for WindowContent.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty WindowContentProperty =
            System.Windows.DependencyProperty.Register("WindowContent", typeof(object), typeof(RibbonApplicationMenu));

        
        #endregion

        #region Depenency Property Changed Callbacks
        
        private static void OnRibbonBarPropertyChanged(System.Windows.DependencyObject o, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
            if (e.OldValue == null && e.NewValue != null) 
            {
                RibbonApplicationMenu menu = o as RibbonApplicationMenu;
                RibbonBar bar = e.NewValue as RibbonBar;

                System.Windows.Data.BindingOperations.SetBinding
                    (
                    menu.MenuItemContent,
                    System.Windows.Controls.ContentPresenter.ContentProperty,
                    new System.Windows.Data.Binding("SelectedItem.Content") { Source = menu, Mode= System.Windows.Data.BindingMode.TwoWay});
            }
        }
        
        #endregion

        #region Overrides

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            this._root = this.GetTemplateChild(PART_Root) as RibbonToggleButton;
            if (this._root != null)
            {
                this._root.Checked += new System.Windows.RoutedEventHandler(this._root_Checked);
                this._root.UnChecked += new System.Windows.RoutedEventHandler(this._root_UnChecked);
            }
            foreach (ApplicationMenuItem item in this.MenuItems)
            {
                if (item.Size == RibbonControlSize.Large)
                {
                    item.Click += this.MenuItemClicked;
                    if (item.IsSelected) 
                    {
                        this.ClearValue(RibbonApplicationMenu.SelectedItemProperty);
                        this.SetValue(RibbonApplicationMenu.SelectedItemProperty, item);
                    }
                }
                this.ItemsOwner.Items.Add(item);
            }
        }

        #endregion

        #region Private Utilities
        #endregion

        #region Event Handlers
        private void MenuItemClicked(object sender, System.Windows.RoutedEventArgs e) 
        {
            ApplicationMenuItem item = sender as ApplicationMenuItem;
            this.SelectItem(item);
        }

        private void SelectItem(ApplicationMenuItem item) 
        {
            ApplicationMenuItem current = this.GetValue(RibbonApplicationMenu.SelectedItemProperty) as ApplicationMenuItem;
            if (current != item)
            {
                item.IsSelected = true;
                this.ClearValue(RibbonApplicationMenu.SelectedItemProperty);
                this.SetValue(RibbonApplicationMenu.SelectedItemProperty, item);
                if (current != null)
                {
                    current.IsSelected = false;
                }
            }
        }

        
        private void _root_Checked(object sender, System.Windows.RoutedEventArgs e) 
        {
            if (this.RibbonBar != null) 
            {
                this.RibbonBar.SetValue(Ribbon2010.RibbonBar.IsApplicationMenuOpenProperty, true);
            }
        }

        private void _root_UnChecked(object sender, System.Windows.RoutedEventArgs e) 
        {
            if (this.RibbonBar != null)
            {
                this.RibbonBar.SetValue(Ribbon2010.RibbonBar.IsApplicationMenuOpenProperty, false);
            }
        }
       
        #endregion

        #region Fields
        private const string PART_Root = "PART_Root";
        internal RibbonToggleButton _root;
        internal System.Windows.Controls.Border Border { get; set; }
        private System.Windows.Controls.ItemsControl ItemsOwner { get; set; }
        private int largeCount = 0;
        private System.Windows.Controls.ContentPresenter MenuItemContent;
        #endregion
        #region IRibbonControl Implementation
        public Interfaces.IRibbonControl Clone()
        {
            throw new NotImplementedException();
        }


        public bool IsLarge
        {
            get { return true; }
        }


        public bool IsClonedItem
        {
            get { return false; }
        }

        public bool IsCloneable
        {
            get { return false; }
        }


        public void DisposeControl()
        {
            throw new NotImplementedException();
        }
        public Interfaces.IRibbonControl ParentControl
        {
            get { return null; }
        }
        #endregion
    }
}
