﻿
namespace Masterpiece.Controls.Ribbon2010
{
    [System.Windows.Localizability(System.Windows.LocalizationCategory.ComboBox)]
    [System.Windows.TemplatePart(Name=PART_Popup, Type=typeof(System.Windows.Controls.Primitives.Popup))]
    [System.Windows.TemplatePart(Name = PART_EditableTextBox, Type = typeof(System.Windows.Controls.TextBox))]
    [System.Windows.StyleTypedProperty(Property = "ItemContainerStyle", StyleTargetType = typeof(RibbonComboBoxItem))]
    public class RibbonComboBox : System.Windows.Controls.ComboBox,   Interfaces.IRibbonControl
    {
        #region Constructors

        static RibbonComboBox() 
        {

            System.Type ownerType = typeof(RibbonComboBox);
            RibbonComboBox.DefaultStyleKeyProperty.OverrideMetadata(ownerType, new System.Windows.FrameworkPropertyMetadata(ownerType));
            //
            RibbonComboBox.ItemsHeaderProperty.OverrideMetadata(ownerType, new System.Windows.FrameworkPropertyMetadata(null, new System.Windows.CoerceValueCallback(RibbonComboBox.CoerceItemsHeaderProperty)));

            RibbonComboBox.MaxDropDownHeightProperty.OverrideMetadata(ownerType, new System.Windows.FrameworkPropertyMetadata(System.Windows.SystemParameters.PrimaryScreenHeight / 1.1));
            
            RibbonComboBox.IsDropDownOpenProperty.OverrideMetadata(ownerType, new System.Windows.FrameworkPropertyMetadata(false, new System.Windows.PropertyChangedCallback(RibbonComboBox.OnIsDropDownOpenPropertyChanged)));
        }
        public RibbonComboBox():base() 
        {
            if (this.IsPopupVisible)
            {
                this.ExpanderCollection = new ExpanderCollection();
            }
        }

        #endregion

        #region Properties and Depenency Properties

        [System.ComponentModel.Bindable(true),
        System.ComponentModel.Browsable(false),
        System.ComponentModel.Category("Appearence")]
        public bool PopupCanResize
        {
            get { return (bool)GetValue(PopupCanResizeProperty); }
            set { SetValue(PopupCanResizeProperty, value); }
        }

        // Using a DependencyProperty as the backing store for PopupCanResize.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty PopupCanResizeProperty =
            System.Windows.DependencyProperty.Register("PopupCanResize", typeof(bool), typeof(RibbonComboBox), new System.Windows.PropertyMetadata(true, new System.Windows.PropertyChangedCallback(RibbonComboBox.OnPopupCanResizePropertyChanged)));

        [System.ComponentModel.Bindable(true),
        System.ComponentModel.Browsable(false),
        System.ComponentModel.Category("Appearence")]
        public string ItemsHeader
        {
            get { return (string)GetValue(ItemsHeaderProperty); }
            set { SetValue(ItemsHeaderProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ItemsHeader.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty ItemsHeaderProperty =
            System.Windows.DependencyProperty.Register("ItemsHeader", typeof(string), typeof(RibbonComboBox));

        [System.ComponentModel.Bindable(true),
        System.ComponentModel.Browsable(false)]
        public bool HasHeader
        {
            get { return (bool)GetValue(HasHeaderProperty); }
        }

        // Using a DependencyProperty as the backing store for HasHeader.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyPropertyKey HasHeaderPropertyKey =
            System.Windows.DependencyProperty.RegisterReadOnly("HasHeader", typeof(bool), typeof(RibbonComboBox), new System.Windows.PropertyMetadata(false));

        internal static readonly System.Windows.DependencyProperty HasHeaderProperty = RibbonComboBox.HasHeaderPropertyKey.DependencyProperty;

         [System.ComponentModel.Bindable(true),
        System.ComponentModel.Browsable(false)]
        public bool HasItemsSource
        {
            get { return (bool)GetValue(HasItemsSourceProperty); }
        }

        // Using a DependencyProperty as the backing store for HasItemsSource.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyPropertyKey HasItemsSourcePropertyKey =
            System.Windows.DependencyProperty.RegisterReadOnly("HasItemsSource", typeof(bool), typeof(RibbonComboBox), new System.Windows.PropertyMetadata(false));

        internal static readonly System.Windows.DependencyProperty HasItemsSourceProperty = RibbonComboBox.HasItemsSourcePropertyKey.DependencyProperty;

        [System.ComponentModel.Bindable(true),
        System.ComponentModel.Browsable(false)]
        public ExpanderCollection ExpanderCollection
        {
            get { return (ExpanderCollection)GetValue(ExpanderCollectionProperty); }
            set { SetValue(ExpanderCollectionProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ExpanderCollection.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty ExpanderCollectionProperty =
            System.Windows.DependencyProperty.Register("ExpanderCollection", typeof(ExpanderCollection), typeof(RibbonComboBox), new System.Windows.PropertyMetadata((ExpanderCollection)null));

        [System.ComponentModel.Bindable(true),
        System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Layout")]
        public double DropDownHeight
        {
            get { return (double)GetValue(DropDownHeightProperty); }
            set { SetValue(DropDownHeightProperty, value); }
        }

        public static readonly System.Windows.DependencyProperty DropDownHeightProperty = RibbonPopupMenu.DropDownHeightProperty.AddOwner(typeof(RibbonComboBox));

        [System.ComponentModel.Bindable(true),
       System.ComponentModel.Browsable(true),
       System.ComponentModel.Category("Text")]
        public string Title
        {
            get { return (string)GetValue(TitleProperty); }
            set { SetValue(TitleProperty, value); }
        }

        public static readonly System.Windows.DependencyProperty TitleProperty = RibbonSplitButton.TitleProperty.AddOwner(typeof(RibbonComboBox));

        [System.ComponentModel.Bindable(true),
       System.ComponentModel.Browsable(true),
       System.ComponentModel.Category("Appearance")]
        public System.Windows.Media.ImageSource Image
        {
            get { return (System.Windows.Media.ImageSource)GetValue(ImageProperty); }
            set { SetValue(ImageProperty, value); }
        }

        public static readonly System.Windows.DependencyProperty ImageProperty = RibbonGroup.ImageProperty.AddOwner(typeof(RibbonComboBox));

        [System.ComponentModel.Bindable(false),
        System.ComponentModel.Browsable(false),
        System.ComponentModel.ReadOnly(false)]
        public int? ItemID
        {
            get { return (int?)GetValue(ItemIDProperty); }
            set { SetValue(ItemIDProperty, value); }
        }

        public static readonly System.Windows.DependencyProperty ItemIDProperty = 
            RibbonButton.ItemIDProperty.AddOwner(typeof(RibbonComboBox), 
            new System.Windows.PropertyMetadata(
                new System.Windows.PropertyChangedCallback(RibbonComboBox.OnItemIDPropertyChanged)));

        internal bool IsPopupVisible
        {
            get { return (bool)GetValue(IsPopupVisibleProperty); }
            set { SetValue(IsPopupVisibleProperty, value); }
        }

        // Using a DependencyProperty as the backing store for IsPopupVisible.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty IsPopupVisibleProperty =
            System.Windows.DependencyProperty.Register("IsPopupVisible", typeof(bool), typeof(RibbonComboBox), new System.Windows.PropertyMetadata(true));
        
        #endregion

        #region Depenency Property Changed Callbacks

        private static void OnIsDropDownOpenPropertyChanged(System.Windows.DependencyObject d, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
            RibbonComboBox rcb = (RibbonComboBox)d;
            bool newValue = (bool)e.NewValue;
            //if (rcb.IsClonedItem)
            //{
            //    if (newValue)
            //    {
            //        rcb.ParentUI.Popup.PlacementTarget = rcb;
            //        rcb.ParentUI.Popup.IsOpen = true;
            //    }
            //}
        }


        private static void OnItemIDPropertyChanged(System.Windows.DependencyObject d, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
            int? id = (int?)e.NewValue;
            if (id.HasValue)
            {
                RibbonControlCloningService.Add(d as Interfaces.IRibbonControl, id.Value);
            }
        }

        private static void OnPopupCanResizePropertyChanged(System.Windows.DependencyObject d, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
            bool newValue = (bool)e.NewValue;
            RibbonComboBox rcb = (RibbonComboBox)d;
            if (newValue)
            {
                if (rcb.Thumb != null) 
                {
                    rcb.Thumb.DragDelta -= new Controls.Primitives.DragDeltaEventHandler(rcb.ThumbDragDelta);
                    rcb.Thumb.DragDelta += new Controls.Primitives.DragDeltaEventHandler(rcb.ThumbDragDelta);
                } 
            }
            else 
            {
                if (rcb.Thumb != null) 
                {
                    rcb.Thumb.DragDelta -= new Controls.Primitives.DragDeltaEventHandler(rcb.ThumbDragDelta);
                }
            }
        }

        private static System.Object CoerceItemsHeaderProperty(System.Windows.DependencyObject d, System.Object value) 
        {
            RibbonComboBox rcb = (RibbonComboBox)d;
            if (value is string)
            {
                string header = (string)value;
                rcb.ClearValue(RibbonComboBox.HasHeaderPropertyKey);
                rcb.SetValue(RibbonComboBox.HasHeaderPropertyKey, !string.IsNullOrEmpty(header));
                return value;
            }
            else 
            {
                rcb.SetValue(RibbonComboBox.HasHeaderPropertyKey, false);
                return null;
            }
        }

        private static System.Object CoerceItemSourceProperty(System.Windows.DependencyObject d, System.Object value)
        {
            RibbonComboBox rcb = (RibbonComboBox)d;
            
            rcb.SetValue(RibbonComboBox.HasItemsSourcePropertyKey, value != null);

            return value;
        }
        
        #endregion

        #region Overrides

        protected override void OnPreviewMouseRightButtonUp(System.Windows.Input.MouseButtonEventArgs e)
        {
            if (Common.PopupService.IsRibbonQAInitialized)
            {
                if (this.ParentUI != null)
                {
                    Common.PopupService.EnableRemovalButton();
                }
                else
                {
                    Common.PopupService.DisableRemovalButton();
                }

                Common.PopupService._addItem.IsEnabled = this.IsCloneable;
                Common.PopupService.OpenContextMenu(this);
            }
            base.OnPreviewMouseRightButtonUp(e);

        }
       
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            this.ContentPresenter = this.GetTemplateChild("ContentSite") as System.Windows.Controls.ContentPresenter;
            this.RootGrid = this.GetTemplateChild(PART_RootGrid) as System.Windows.Controls.Grid;
            this.Popup = this.GetTemplateChild("PART_Popup") as System.Windows.Controls.Primitives.Popup;
            this.ToggleButton = this.GetTemplateChild(PART_Toggle) as System.Windows.Controls.Primitives.ToggleButton;
            this.EditableTextBox = this.GetTemplateChild("PART_EditableTextBox") as System.Windows.Controls.TextBox;
            if (this.IsPopupVisible)
            {
                this.ExtraItems = this.GetTemplateChild(PART_ExtraItems) as System.Windows.Controls.StackPanel;
                if (this.Thumb != null)
                {
                    this.Thumb.DragDelta -= new Controls.Primitives.DragDeltaEventHandler(this.ThumbDragDelta);
                }
                this.Thumb = this.GetTemplateChild(PART_Thumb) as Controls.Primitives.Thumb;
                
                this._border = this.GetTemplateChild("PART_Border") as System.Windows.Controls.Border;

                if (this.PopupCanResize)
                {
                    this.RootGrid.Height = this.DropDownHeight;
                    this.Thumb.DragDelta += new Controls.Primitives.DragDeltaEventHandler(this.ThumbDragDelta);
                }

                foreach (RibbonHeaderedExpander item in this.ExpanderCollection)
                {
                    item.IsExpandable = false;
                    System.Windows.Data.BindingOperations.SetBinding
                        (
                         item,
                            RibbonHeaderedExpander.ItemContainerStyleProperty,
                            new System.Windows.Data.Binding("ItemContainerStyle") { Source = this, Mode = System.Windows.Data.BindingMode.TwoWay }
                        );
                    this.ExtraItems.Children.Add(item);
                }
            }
            else 
            {
                if (this.Popup != null) 
                {
                    Masterpiece.Controls.DropShadowChrome shadow = this.Popup.Child as Masterpiece.Controls.DropShadowChrome;
                    
                    if (shadow != null) 
                    {
                        shadow.Visibility = System.Windows.Visibility.Collapsed;
                        shadow = null;
                    }
                }
                
                System.Windows.Data.BindingOperations.SetBinding
                   (
                       this.ContentPresenter,
                       System.Windows.Controls.ContentPresenter.ContentProperty,
                       new System.Windows.Data.Binding("Content") { Source = this.ParentUI.ContentPresenter, Mode = System.Windows.Data.BindingMode.TwoWay }
                   );
                System.Windows.Data.BindingOperations.SetBinding
                  (
                      this,
                      RibbonComboBox.TextProperty,
                      new System.Windows.Data.Binding("Text") { Source = this.ParentUI, Mode = System.Windows.Data.BindingMode.TwoWay }
                  );
                if (this.RootGrid != null) 
                {
                    this.RootGrid.Height = 20;
                }
            }
            
        }

        protected override void OnDropDownOpened(System.EventArgs e)
        {
            base.OnDropDownOpened(e);
            this.Popup.PlacementTarget = this;
            if (this.RootGrid != null && this.PopupCanResize)
            {
                this.RootGrid.SetValue(System.Windows.Controls.Grid.HeightProperty, this.DropDownHeight);
            }
        }

        protected override void OnDropDownClosed(System.EventArgs e)
        {
            base.OnDropDownClosed(e);
            if (Common.PopupService.RibbonBar != null && 
                Common.PopupService.RibbonBar.IsBackgroundMode)
            {
                Common.PopupService.RaisePopupDismissedEvent(this);
            }

            if (this.ClonedChild != null && 
                this.ClonedChild.ToggleButton != null && 
                (bool)this.ClonedChild.ToggleButton.IsChecked)
            {
                this.ClonedChild.ToggleButton.IsChecked = false;
            }
        }
       
        protected override System.Windows.DependencyObject GetContainerForItemOverride()
        {
            return new RibbonComboBoxItem();
        }

        protected override bool IsItemItsOwnContainerOverride(object item)
        {
            return item is RibbonComboBoxItem;
        }
       
        #endregion

        #region Private Utilities

        //private void SetPopupIsOpenProperty(bool value) 
        //{
        //    if (!this.IsClonedItem)
        //    {
        //        if (this.Popup != null)
        //        {
        //            this.Popup.PlacementTarget = this;
        //            this.Popup.IsOpen = value;
        //        }
        //    }
        //    else 
        //    {
        //        if (this.ParentUI != null && this.ParentUI.Popup != null) 
        //        {
        //            this.ParentUI.Popup.PlacementTarget = this;
        //            this.ParentUI.Popup.IsOpen = value;
        //        }
        //    }
        //}
        #endregion

        #region Event Handlers
        
        private void ThumbDragDelta(object sender, Controls.Primitives.DragDeltaEventArgs e) 
        {
            double height = this.RootGrid.ActualHeight;
            e.Handled = true;

            if (System.Double.IsNaN(height) || System.Double.IsPositiveInfinity(height)) 
            {
                return;
            }

            height += e.VerticalChange;
            if (height <= this.RootGrid.MinHeight) 
            {
                return;
            }
            this.RootGrid.Height = height;
        }

        #endregion

        #region Fields
        
        private const string PART_Toggle = "PART_ToggleButton";
        private const string PART_Popup = "PART_Popup";
        private const string PART_EditableTextBox = "PART_EditableTextBox";
        private const string PART_Thumb = "PART_Thumb";
        private const string PART_ExtraItems = "PART_ExtraItems";
        private const string PART_RootGrid = "PART_RootGrid";
        private const string PART_Border = "PART_Border";

        private System.Windows.Controls.StackPanel ExtraItems;
        private Controls.Primitives.Thumb Thumb;
        private System.Windows.Controls.Grid RootGrid;
        private System.Windows.Controls.Border _border;
        private System.Windows.Size infiniteSize = new System.Windows.Size(double.PositiveInfinity, double.PositiveInfinity);
        private bool isCloneable = false, isClonedItem = false;
        private RibbonComboBox ParentUI, ClonedChild;
        internal System.Windows.Controls.ContentPresenter ContentPresenter;
        private System.Windows.Controls.Primitives.Popup Popup;
        internal System.Windows.Controls.Primitives.ToggleButton ToggleButton;
        internal System.Windows.Controls.TextBox EditableTextBox;
        #endregion
        
        #region IRibbonControl Implementation
        
        public bool IsLarge
        {
            get { return false; }
        }

        public Interfaces.IRibbonControl Clone()
        {
            RibbonComboBox rcb = new RibbonComboBox() { IsPopupVisible = false };
            rcb.IsClonedItem = true;
            rcb.ParentUI = this;
            System.Windows.Data.BindingOperations.SetBinding
                (
                    rcb,
                    RibbonComboBox.IsEditableProperty,
                    new System.Windows.Data.Binding("IsEditable") { Source = this, Mode = System.Windows.Data.BindingMode.TwoWay }
                );
            this.ClonedChild = rcb;
            return rcb;
        }

        public bool IsClonedItem
        {
            get { return this.isClonedItem; }
            internal set
            {
                this.isClonedItem = value;
            }
        }

        public bool IsCloneable
        {
            get
            {
                return !(this.ClonedChild != null || !RibbonControlCloningService.CheckIfItemIsCloneable(this));
            }
            internal set { this.isCloneable = value; }
        }

        public void DisposeControl()
        {
            if (this.ClonedChild != null) 
            {
                System.Windows.Data.BindingOperations.ClearAllBindings(this.ClonedChild);
                this.ClonedChild = null;
                System.GC.Collect();
            }
        }

        public Interfaces.IRibbonControl ParentControl
        {
            get { return this.ParentUI; }
        }
        #endregion
    }
}
