﻿namespace Masterpiece.Controls.Ribbon2010
{
    [System.Windows.TemplatePart(Name = PART_Scroller, Type = typeof(RibbonScrollViewer))]
    [System.Windows.TemplatePart(Name = PART_Panel, Type = typeof(RibbonWrapPanel))]
    [System.Windows.TemplatePart(Name = PART_Toggle, Type = typeof(RibbonToggleButton))]
    [System.Windows.StyleTypedProperty(Property = "ItemContainerStyle", StyleTargetType = typeof(RibbonGalleryItem))]
    public partial class RibbonGallery : System.Windows.Controls.Primitives.Selector, Interfaces.IRibbonControl
    {
        #region Constructors

        static RibbonGallery() 
        {
            System.Type ownerType=typeof(RibbonGallery);
           RibbonGallery.DefaultStyleKeyProperty.OverrideMetadata(
                    ownerType,
                    new System.Windows.FrameworkPropertyMetadata(ownerType)
                );
        }

        public RibbonGallery()
            : base()
        {
            this.PopupMenu = new RibbonPopupMenu();
        }
        #endregion

        #region Properties and Depenency Properties



        public System.Windows.Style DropDownItemContainerStyle
        {
            get { return (System.Windows.Style)GetValue(DropDownItemContainerStyleProperty); }
            set { SetValue(DropDownItemContainerStyleProperty, value); }
        }

        // Using a DependencyProperty as the backing store for DropDownItemContainerStyle.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty DropDownItemContainerStyleProperty =
            System.Windows.DependencyProperty.Register("DropDownItemContainerStyle", typeof(System.Windows.Style), typeof(RibbonGallery), new System.Windows.PropertyMetadata((System.Windows.Style)null));


        public System.Windows.DataTemplate DropDownItemTemplate
        {
            get { return (System.Windows.DataTemplate)GetValue(DropDownItemTemplateProperty); }
            set { SetValue(DropDownItemTemplateProperty, value); }
        }

        // Using a DependencyProperty as the backing store for DropDownItemTemplate.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty DropDownItemTemplateProperty =
            System.Windows.DependencyProperty.Register("DropDownItemTemplate", typeof(System.Windows.DataTemplate), typeof(RibbonGallery), new System.Windows.PropertyMetadata((System.Windows.DataTemplate)null));

        [System.ComponentModel.Bindable(true),
        System.ComponentModel.Browsable(false),
        System.ComponentModel.ReadOnly(false)]
        public int WrapBy
        {
            get { return (int)GetValue(WrapByProperty); }
            set { SetValue(WrapByProperty, value); }
        }

        // Using a DependencyProperty as the backing store for WrapBy.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty WrapByProperty =
            System.Windows.DependencyProperty.Register("WrapBy", typeof(int), typeof(RibbonGallery), new System.Windows.PropertyMetadata(3));
        
        public double ScrollBy
        {
            get { return (double)GetValue(ScrollByProperty); }
            set { SetValue(ScrollByProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ScrollBy.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty ScrollByProperty =
            System.Windows.DependencyProperty.Register("ScrollBy", typeof(double), typeof(RibbonGallery), new System.Windows.PropertyMetadata(23.0D));

        internal RibbonControlSize Size
        {
            get { return (RibbonControlSize)GetValue(SizeProperty); }
            set { SetValue(SizeProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Size.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty SizeProperty =
            System.Windows.DependencyProperty.Register("Size", typeof(RibbonControlSize), typeof(RibbonGallery), new System.Windows.PropertyMetadata(RibbonControlSize.Large));

        internal int PreviousSelectedIndex
        {
            get { return (int)GetValue(PreviousSelectedIndexProperty); }
            set { SetValue(PreviousSelectedIndexProperty, value); }
        }

        // Using a DependencyProperty as the backing store for PreviousSelectedIndex.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty PreviousSelectedIndexProperty =
            System.Windows.DependencyProperty.Register("PreviousSelectedIndex", typeof(int), typeof(RibbonGallery), new System.Windows.PropertyMetadata(-1, new System.Windows.PropertyChangedCallback(RibbonGallery.OnPreviousSelectedIndexPropertyChanged)));
        
        [System.ComponentModel.Bindable(true)]
        public PopupMenuItemCollection PopupMenuItems 
        {
            get { return this.PopupMenu.Items; }
        }

        public bool IsDropDownOpen
        {
            get { return (bool)GetValue(IsDropDownOpenProperty); }
            set { SetValue(IsDropDownOpenProperty, value); }
        }
        public static readonly System.Windows.DependencyProperty IsDropDownOpenProperty = RibbonComboBox.IsDropDownOpenProperty.AddOwner
            (
            typeof(RibbonGallery), 
            new System.Windows.FrameworkPropertyMetadata
                (
                false, 
                new System.Windows.PropertyChangedCallback(
                    RibbonGallery.OnIsDropDownOpenPropertyChanged), 
                    new System.Windows.CoerceValueCallback(RibbonGallery.CoerceIsDropDownOpenProperty)));
        
        #endregion

        #region Depenency Property Changed Callbacks

        private static System.Object CoerceIsDropDownOpenProperty(System.Windows.DependencyObject o, System.Object value) 
        {
            bool newValue = (bool)value;
            RibbonGallery gallery = (RibbonGallery)o;
            if (newValue) 
            {
                Common.PopupService.OpeningRibbonControl = gallery;
                Common.PopupService.RibbonControlOpening = true;
            }
            return value;
        }

        private static void OnIsDropDownOpenPropertyChanged(System.Windows.DependencyObject o, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
            bool newValue = (bool)e.NewValue, oldValue = !newValue;
            RibbonGallery gallery = (RibbonGallery)o;
            if (!newValue)
            {
                if (gallery.ShadowChrome != null)
                {
                    System.Windows.ValueSource source = System.Windows.DependencyPropertyHelper.GetValueSource(gallery.ShadowChrome, Controls.DropShadowChrome.OpacityProperty);
                    if (source.IsAnimated)
                    {
                        gallery.ShadowChrome.Opacity = 1.0D;
                    }
                }
            }
        }
        private static void OnPreviousSelectedIndexPropertyChanged(System.Windows.DependencyObject d, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
            RibbonGallery gallery = (RibbonGallery)d;
            int previousIndex = (int)e.NewValue;
            if (previousIndex != gallery.SelectedIndex) 
            {
                System.Windows.Controls.ItemContainerGenerator generator = gallery.ItemContainerGenerator;
                RibbonGalleryItem item = generator.ContainerFromIndex(previousIndex) as RibbonGalleryItem;
                if (item != null)
                {
                    item.IsSelected = false;
                }
            }
        }

        #endregion

        #region Overrides

        protected override void OnSelectionChanged(System.Windows.Controls.SelectionChangedEventArgs e)
        {
            base.OnSelectionChanged(e);
            this.SelectItem();
            this.DeselectItem();
            this.MakeVisibleSelectedItem(this.SelectedIndex);
        }

      
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
                       
            if (this.scroller != null) 
            {
                this.scroller = null;
            }
            
            this.scroller = (RibbonScrollViewer)this.GetTemplateChild("PART_Scroller");
            
            if (this.PART_StackPanel == null) 
            {
                this.PART_StackPanel = (RibbonStackPanel)this.GetTemplateChild("PART_StackPanel");
            }
            this._PART_Panel = this.GetTemplateChild(PART_Panel) as RibbonWrapPanel;
            
            System.ComponentModel.DependencyPropertyDescriptor desc;
            if (this._PART_Panel != null)
            {
                this.BuildPopup();
            }
            if (this._toggle != null) 
            {
                /*Removing previous handle of toggle button*/
                this._toggle.Checked -= new System.Windows.RoutedEventHandler(this.Toggle_Checked);
                this._toggle.UnChecked -= new System.Windows.RoutedEventHandler(this.Toggle_Unchecked);
                /*Disposing object manually*/
                this._toggle = null;
            }
            this._toggle = this.GetTemplateChild(PART_Toggle) as RibbonToggleButton;
            if (this._toggle != null) 
            {
                this._toggle.Checked += new System.Windows.RoutedEventHandler(this.Toggle_Checked);
                this._toggle.UnChecked += new System.Windows.RoutedEventHandler(this.Toggle_Unchecked);
            }
            Common.PopupService.PopupDismissedEvent -= this.DissmisPopupEventRaised;
            Common.PopupService.PopupDismissedEvent += new System.EventHandler(this.DissmisPopupEventRaised);
            Common.PopupService.RibbonGalleryItemSelectedEvent -= this.OnRibbonGalleryItemClicked;
            Common.PopupService.RibbonGalleryItemSelectedEvent += this.OnRibbonGalleryItemClicked;
            
        }

        protected override System.Windows.DependencyObject GetContainerForItemOverride()
        {
            return new RibbonGalleryItem();
        }

        protected override bool IsItemItsOwnContainerOverride(object item)
        {
            return item is RibbonGalleryItem;
        }

        protected override void OnItemsSourceChanged(System.Collections.IEnumerable oldValue, System.Collections.IEnumerable newValue)
        {
            base.OnItemsSourceChanged(oldValue, newValue);
        }

        #endregion

        #region Private Utilities

        private void SelectItem()
        {
            if (this.ItemsSource != null)
            {
                if (this.SelectedIndex >= 0)
                {
                    RibbonGalleryItem selectedItem = this.ItemContainerGenerator.ContainerFromIndex(this.SelectedIndex) as RibbonGalleryItem;
                    if (selectedItem != null)
                    {
                        selectedItem.IsSelected = true;
                    }
                }
            }
            else
            {
                if (this.SelectedIndex >= 0)
                {
                    RibbonGalleryItem selectedItem = this.Items[this.SelectedIndex] as RibbonGalleryItem;
                    if (selectedItem != null)
                    {
                        selectedItem.IsSelected = true;
                    }
                }
            }
        }

        private void DeselectItem()
        {
            if (this.ItemsSource != null)
            {
                if (this.PreviousSelectedIndex >= 0)
                {
                    RibbonGalleryItem pItem = this.ItemContainerGenerator.ContainerFromIndex(this.PreviousSelectedIndex) as RibbonGalleryItem;
                    if (pItem != null)
                    {
                        pItem.IsSelected = false;
                    }
                }
            }
            else
            {
                if (this.PreviousSelectedIndex >= 0)
                {
                    RibbonGalleryItem selectedItem = this.Items[this.PreviousSelectedIndex] as RibbonGalleryItem;
                    if (selectedItem != null)
                    {
                        selectedItem.IsSelected = false;
                    }
                }
            }
        }

        private void BuildPopup() 
        {
            if (this.Popup == null) 
            {
                this.Popup = new System.Windows.Controls.Primitives.Popup();
                this.Popup.AllowsTransparency = true;
                this.Popup.Placement = System.Windows.Controls.Primitives.PlacementMode.Bottom;
                this.Popup.PopupAnimation = System.Windows.Controls.Primitives.PopupAnimation.Fade;
                this.Popup.HorizontalOffset = this.WrapBy * (-72) - 6;
                this.Popup.StaysOpen = false;
                this.ShadowChrome = new DropShadowChrome();
                this.ShadowChrome.SnapsToDevicePixels = true;
                this.Expander = new RibbonHeaderedExpander() { IsParentRibbonGallery = true };
                this.Expander.WrapBy = this.WrapBy;
                
                if (this.ItemsSource == null)
                {
                    foreach (var item in this.Items)
                    {
                        if (item is RibbonGalleryItem)
                        {
                            RibbonGalleryItem galleryItem=item as RibbonGalleryItem;
                            RibbonHeaderedExpanderItem headerItem = new RibbonHeaderedExpanderItem() {Image=galleryItem.Image };
                            headerItem.Style = this.DropDownItemContainerStyle;
                            this.Expander.Items.Add(headerItem);
                        }
                    }
                }
                else 
                {
                    /*
                      System.Collections.ArrayList items = new System.Collections.ArrayList();
                    System.Collections.IEnumerator enums = this.ItemsSource.GetEnumerator();
                    while (enums.MoveNext()) 
                    {
                        items.Add(enums.Current);
                    }
                    this.Expander.ItemsSource = items;
                     */
                    System.Windows.Data.BindingOperations.SetBinding
                        (
                            this.Expander,
                            RibbonHeaderedExpander.ItemsSourceProperty,
                            new System.Windows.Data.Binding("ItemsSource") { Source = this, Mode = System.Windows.Data.BindingMode.TwoWay }
                        );
                    
                    System.Windows.Data.BindingOperations.SetBinding
                        (
                            this.Expander,
                            RibbonHeaderedExpander.ItemTemplateProperty,
                            new System.Windows.Data.Binding("DropDownItemTemplate") { Source = this, Mode = System.Windows.Data.BindingMode.TwoWay }
                        );
                    System.Windows.Data.BindingOperations.SetBinding
                        (
                            this.Expander,
                            RibbonHeaderedExpander.ItemContainerStyleProperty,
                            new System.Windows.Data.Binding("DropDownItemContainerStyle") { Source = this, Mode = System.Windows.Data.BindingMode.TwoWay }
                        );
                   
                }
                System.Windows.Data.BindingOperations.SetBinding
                       (
                           this.Expander,
                           RibbonHeaderedExpander.SelectedIndexProperty,
                           new System.Windows.Data.Binding("SelectedIndex") { Source = this, Mode = System.Windows.Data.BindingMode.TwoWay }
                       );

                System.Windows.Data.BindingOperations.SetBinding
                   (
                       this.Expander,
                       RibbonHeaderedExpander.PreviousSelectedIndexProperty,
                       new System.Windows.Data.Binding("PreviousSelectedIndex") { Source = this, Mode = System.Windows.Data.BindingMode.TwoWay }
                   );
                this.PopupMenu.ExpanderItems.Add(this.Expander);
                this.ShadowChrome.Content = this.PopupMenu;
               // shadowChrome.Opacity = 0.0;
                this.Popup.Child = this.ShadowChrome;
                System.Windows.Data.BindingOperations.SetBinding
                    (
                         this,
                         RibbonGallery.IsDropDownOpenProperty,
                         new System.Windows.Data.Binding("IsOpen") { Source = this.Popup, Mode = System.Windows.Data.BindingMode.TwoWay }
                    );
                System.Windows.Data.BindingOperations.SetBinding
                    (
                         this.PopupMenu,
                         RibbonPopupMenu.IsDropDownOpenProperty,
                         new System.Windows.Data.Binding("IsDropDownOpen") { Source = this, Mode = System.Windows.Data.BindingMode.TwoWay }
                    );
                if (this.StoryBoard == null) 
                {
                    this.StoryBoard = new System.Windows.Media.Animation.Storyboard();
                    this.DoubleAnimation = new System.Windows.Media.Animation.DoubleAnimation();
                    this.DoubleAnimation.Duration = new System.Windows.Duration(System.TimeSpan.FromSeconds(0.38));
                    System.Windows.Media.Animation.Storyboard.SetTarget(this.DoubleAnimation, this.ShadowChrome);
                    System.Windows.Media.Animation.Storyboard.SetTargetProperty(this.DoubleAnimation, new System.Windows.PropertyPath("Opacity"));
                    this.StoryBoard.Completed += new System.EventHandler(this.ClosingPopupCompleted);
                    this.StoryBoard.Children.Add(this.DoubleAnimation);
                    this.StoryBoard.FillBehavior = System.Windows.Media.Animation.FillBehavior.Stop;
                    
                }
            }
        }

        internal void DetachClickEventHandler() 
        {
            if (this.scroller != null) 
            {
                this.scroller.button.Click -= this.OnPopupOpenButtonClicked;
            }
        }

        internal void OnScrollViewewInitialized() 
        {
            this.scroller.button.Click += new RoutedEventHandler(this.OnPopupOpenButtonClicked);
        }

        internal void AnimatePopupContent(double from, double to) 
        {
            if (this.StoryBoard != null) 
            {
                this.DoubleAnimation.From = from;
                this.DoubleAnimation.To = to;
                this.StoryBoard.Begin();
            }
        }

        #endregion

        #region Event Handlers

        private void OnRibbonGalleryItemClicked(object sender, System.EventArgs e) 
        {
            if (sender != null && sender is RibbonGalleryItem) 
            {
                this.PreviousSelectedIndex = this.SelectedIndex;
                RibbonGalleryItem selectedItem = sender as RibbonGalleryItem;
                object selectedObject = this.ItemsSource != null ? selectedItem.DataContext : sender;
                int index = this.Items.IndexOf(selectedObject);
                if (index >= 0)
                {
                    this.SelectedIndex = index;
                }
            }
        }
        private void ClosingPopupCompleted(object sender, System.EventArgs e) 
        {
            this.Popup.PopupAnimation = System.Windows.Controls.Primitives.PopupAnimation.None;
            this.IsDropDownOpen = false;
            this.StoryBoard.ClearValue(DropShadowChrome.OpacityProperty);
            this.Popup.PopupAnimation = System.Windows.Controls.Primitives.PopupAnimation.Fade;
        }

        private void DissmisPopupEventRaised(object sender, System.EventArgs e) 
        {
           this.IsDropDownOpen = false;
        }
        
        private void OnPopupOpenButtonClicked(object sender, Masterpiece.RoutedEventArgs e) 
        {
            if (this.IsDropDownOpen) 
            {
                System.Diagnostics.Debug.WriteLine("Tried to open popup while it has been open. For developer!");
                return;
            }
            this.Popup.PlacementTarget = this.scroller.button;
            this.Popup.VerticalOffset = -65;
            this.IsDropDownOpen = true;
        }
       
        private void Toggle_Checked(object sender, System.Windows.RoutedEventArgs e) 
        {
            //if (this.Popup == null) 
            //{
            //    if (this.ParentUI != null) 
            //    {
            //        if (!this.ParentUI.Popup.IsOpen)
            //        {
            //            this.ParentUI.Popup.PlacementTarget = this._toggle;
            //            this.ParentUI.Popup.HorizontalOffset = 0;
            //            this.ParentUI.Popup.VerticalOffset = 0;
            //            Common.PopupService.OpenPopup(this.ParentUI.Popup, this._toggle);
            //            System.Windows.Input.Mouse.Capture(this, System.Windows.Input.CaptureMode.SubTree);
            //        }
            //        else 
            //        {
            //            this.ParentUI.Popup.IsOpen = false;
            //            this.ParentUI.Popup.HorizontalOffset = 0;
            //            this.ParentUI.Popup.VerticalOffset = 0;
            //            this.ParentUI.Popup.PlacementTarget = this._toggle;
            //            this.ParentUI.Popup.IsOpen = true;
            //            System.Windows.Input.Mouse.Capture(this, System.Windows.Input.CaptureMode.SubTree);
            //        }
            //    }
            //}
        }

        private void Toggle_Unchecked(object sender, System.Windows.RoutedEventArgs e) 
        {
            //if (this.Popup == null)
            //{
            //    if (this.ParentUI != null)
            //    {
            //        if (this.ParentUI.Popup.IsOpen)
            //        {
            //            this.ParentUI.Popup.PlacementTarget = this._toggle;
            //            Common.PopupService.OpenPopup(this.ParentUI.Popup, this._toggle);
            //        }
            //    }
            //}
        }
       
        private void MakeVisibleSelectedItem(int index) 
        {
            System.Windows.Media.TranslateTransform trans = (System.Windows.Media.TranslateTransform)this.PART_StackPanel.RenderTransform;
            double offset = index / this._PART_Panel.WrapBy * this.ScrollBy;
            this.PART_StackPanel.ScrollWithoutAnimation(offset);
            if (this.PopupMenu != null && this.IsDropDownOpen) 
            {
                this.AnimatePopupContent(1.0, 0.0);
            }
        }

        #endregion

        #region Fields

        private const string PART_Scroller = "PART_Scroller";
        private const string PART_Panel = "PART_Panel";
        private const string PART_Toggle = "PART_Toggle";
        internal RibbonScrollViewer scroller;
        internal RibbonWrapPanel _PART_Panel = null;
        private RibbonStackPanel PART_StackPanel;
        private RibbonToggleButton _toggle;
        internal RibbonGallery ParentUI, ClonedChild;
        internal bool IsCloned = false;
        internal System.Windows.Controls.Primitives.Popup Popup;
        internal RibbonHeaderedExpander Expander;
        internal RibbonPopupMenu PopupMenu;
        internal bool ShouldMakeVisible = true;
        internal Controls.DropShadowChrome ShadowChrome;
        internal System.Windows.Media.Animation.Storyboard StoryBoard;
        internal System.Windows.Media.Animation.DoubleAnimation DoubleAnimation;
        private bool isCloneable = false, isClonedItem = false;

        #endregion
        #region IRibbonControl Implementation
        public bool IsLarge
        {
            get { return true; }
        }

        public Interfaces.IRibbonControl Clone()
        {
            RibbonGallery clonedItem = null;
            if (!this.IsCloned && this.ClonedChild == null) 
            {
                this.IsCloned = true;
                clonedItem = new RibbonGallery();
                clonedItem.Size = RibbonControlSize.Small;
                this.ClonedChild = clonedItem;
                clonedItem.ParentUI = this;
            }

            return clonedItem;
        }

        public bool IsClonedItem
        {
            get { return this.isClonedItem; }
        }

        public bool IsCloneable
        {
            get { return this.isCloneable; }
        }


        public void DisposeControl()
        {
            throw new System.NotImplementedException();
        }

        public Interfaces.IRibbonControl ParentControl
        {
            get { return null; }
        }
        #endregion
    }
}
