﻿
namespace Masterpiece.Controls.Ribbon2010
{
    [System.Windows.TemplatePart(Name = PART_Launcher, Type = typeof(System.Windows.Controls.Button))]
    [System.Windows.TemplatePart(Name = PART_Toogle, Type = typeof(Controls.Primitives.ToggleButton))]
    [System.Windows.TemplatePart(Name = PART_Popup, Type = typeof(System.Windows.Controls.Primitives.Popup))]
    public class RibbonGroup : System.Windows.Controls.ItemsControl, Interfaces.IRibbonControl
    {
        #region Constructors
        static RibbonGroup() 
        {
            System.Type ownerType = typeof(RibbonGroup);
            RibbonGroup.DefaultStyleKeyProperty.OverrideMetadata
                (
                ownerType,
                new System.Windows.FrameworkPropertyMetadata(ownerType)
                );

        }

        public RibbonGroup():base() 
        {
        }

        #endregion

        #region Properties and Depenency Properties
        
        [System.ComponentModel.Bindable(true), System.ComponentModel.Browsable(false), System.ComponentModel.Category("Appearance"), System.ComponentModel.ReadOnly(false)]
        public string Title
        {
            get { return (string)GetValue(TitleProperty); }
            set { SetValue(TitleProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Title.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty TitleProperty =
            System.Windows.DependencyProperty.Register("Title", typeof(string), typeof(RibbonGroup), new System.Windows.PropertyMetadata(""));

        [System.ComponentModel.Bindable(true), System.ComponentModel.Browsable(false), System.ComponentModel.Category("Appearance"), System.ComponentModel.ReadOnly(false)]
        public bool HasLauncher
        {
            get { return (bool)GetValue(HasLauncherProperty); }
            set { SetValue(HasLauncherProperty, value); }
        }

        // Using a DependencyProperty as the backing store for HasLauncher.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty HasLauncherProperty =
            System.Windows.DependencyProperty.Register("HasLauncher", typeof(bool), typeof(RibbonGroup), new System.Windows.PropertyMetadata(false));
        
        [System.ComponentModel.Bindable(true), System.ComponentModel.Browsable(false), System.ComponentModel.Category("Appearance"), System.ComponentModel.ReadOnly(true)]
        public bool Minimized
        {
            get { return (bool)GetValue(MinimizedProperty); }
        }

        // Using a DependencyProperty as the backing store for Minimized.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyPropertyKey MinimizedPropertyKey =
            System.Windows.DependencyProperty.RegisterReadOnly("Minimized", typeof(bool), typeof(RibbonGroup), new System.Windows.PropertyMetadata(false));

        internal static readonly System.Windows.DependencyProperty MinimizedProperty = MinimizedPropertyKey.DependencyProperty;

        [System.ComponentModel.Bindable(true), System.ComponentModel.Browsable(false), System.ComponentModel.Category("Appearance"), System.ComponentModel.ReadOnly(true)]
        public bool CanMinimize
        {
            get { return (bool)GetValue(CanMinimizeProperty); }
        }
        // Using a DependencyProperty as the backing store for CanMinimize.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyPropertyKey CanMinimizePropertyKey =
            System.Windows.DependencyProperty.RegisterReadOnly("CanMinimize", typeof(bool), typeof(RibbonGroup), new System.Windows.PropertyMetadata(true));

        internal static readonly System.Windows.DependencyProperty CanMinimizeProperty = CanMinimizePropertyKey.DependencyProperty;

        internal RibbonControlSize Size
        {
            get { return (RibbonControlSize)GetValue(SizeProperty); }
            set { SetValue(SizeProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Size.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty SizeProperty =
            System.Windows.DependencyProperty.Register("Size", typeof(RibbonControlSize), typeof(RibbonGroup), 
            new System.Windows.FrameworkPropertyMetadata(
                RibbonControlSize.Large, 
                new System.Windows.PropertyChangedCallback(RibbonGroup.OnSizePropertyChanged), 
                new System.Windows.CoerceValueCallback(RibbonGroup.CoerceSizeProperty)));

        public System.Windows.Media.ImageSource Image
        {
            get { return (System.Windows.Media.ImageSource)GetValue(ImageProperty); }
            set { SetValue(ImageProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Image.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty ImageProperty =
            System.Windows.DependencyProperty.Register("Image", typeof(System.Windows.Media.ImageSource), typeof(RibbonGroup), new System.Windows.PropertyMetadata((System.Windows.Media.ImageSource)null));

        public bool IsDropDownOpen
        {
            get { return (bool)GetValue(IsDropDownOpenProperty); }
            set { SetValue(IsDropDownOpenProperty, value); }
        }

        // Using a DependencyProperty as the backing store for IsDropDownOpen.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty IsDropDownOpenProperty = RibbonComboBox.IsDropDownOpenProperty.AddOwner(typeof(RibbonGroup), new System.Windows.PropertyMetadata(false, new System.Windows.PropertyChangedCallback(RibbonGroup.OnIsDropDownOpenPropertyChanged), new System.Windows.CoerceValueCallback(RibbonGroup.CoerceIsDropDownOpenProperty)));

        public int? ItemID
        {
            get { return (int?)GetValue(ItemIDProperty); }
            set { SetValue(ItemIDProperty, value); }
        }

        public static readonly System.Windows.DependencyProperty ItemIDProperty = RibbonButton.ItemIDProperty.AddOwner(typeof(RibbonGroup), new System.Windows.PropertyMetadata(new System.Windows.PropertyChangedCallback(RibbonGroup.OnItemIDPropertyChanged)));
        
        #endregion

        #region Depenency Property Changed Callbacks

        private static void OnItemIDPropertyChanged(System.Windows.DependencyObject d, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
            int? id = (int?)e.NewValue;
            if (id.HasValue)
            {
                RibbonControlCloningService.Add(d as Interfaces.IRibbonControl, id.Value);
            }
        }
        private static System.Object CoerceIsDropDownOpenProperty(System.Windows.DependencyObject d, System.Object value) 
        {
            return value;
        }

        private static void OnIsDropDownOpenPropertyChanged(System.Windows.DependencyObject o, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
            bool newValue = (bool)e.NewValue;
            RibbonGroup group = (RibbonGroup)o;
            if (newValue)
            {
                Common.PopupService.OpeningRibbonControl = group;
                Common.PopupService.RibbonControlOpening = true;
            }
            else 
            {
                if (Common.PopupService.OpeningRibbonControl == group) 
                {
                    Common.PopupService.RibbonControlOpening = false;
                }
            }
            group.SetPopupIsOpenProperty(newValue);
        }

        private static void OnSizePropertyChanged(System.Windows.DependencyObject o, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
            RibbonControlSize oldValue = (RibbonControlSize)e.OldValue, newValue = (RibbonControlSize)e.NewValue;
            RibbonGroup group = (RibbonGroup)o;
            if (oldValue == RibbonControlSize.Small && newValue == RibbonControlSize.Large) 
            {
                if (group.ribbonToggle != null)
                {
                    if (group._popup != null)
                    {
                        group.ribbonToggle.Checked -= new System.Windows.RoutedEventHandler(group.RibbonToggleButtonChecked);
                        group.ribbonToggle.UnChecked -= new System.Windows.RoutedEventHandler(group.RibbonToggleButtonUnChecked);
                    }
                    group.ribbonToggle = null;
                }
            }
        }

        private static System.Object CoerceSizeProperty(System.Windows.DependencyObject o, System.Object value) 
        {
            return value is RibbonControlSize ? value : RibbonControlSize.Large;
        }

        private static void OnMinimizedPropertyChanged(System.Windows.DependencyObject o, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
            bool oldValue = (bool)e.OldValue, newValue = (bool)e.NewValue;
            RibbonGroup group = o as RibbonGroup;
        }

        #endregion
        
        #region Overrieds
       
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            if (this.launcher != null) 
            {
                Common.PopupService.PopupDismissedEvent -= this.DismissPopupEventRaised;
            }
            this.launcher = this.GetTemplateChild(PART_Launcher) as System.Windows.Controls.Button;
            if (this.HasLauncher)
            {
                if (this.launcher != null)
                {
                    this.launcher.Click += new System.Windows.RoutedEventHandler(this.launcher_Click);
                }
            }
           
            this._togle = this.GetTemplateChild(PART_Toogle) as Controls.Primitives.ToggleButton;

            if (this._togle != null) 
            {
                this._popup = this.GetTemplateChild(PART_Popup) as System.Windows.Controls.Primitives.Popup;
            }
            
            if (this.ribbonToggle != null)
            {
               this.ribbonToggle.Checked -= this.RibbonToggleButtonChecked;
               this.ribbonToggle.UnChecked -= this.RibbonToggleButtonUnChecked;
            }
            
            this.ribbonToggle = this.GetTemplateChild(PART_Toogle) as RibbonToggleButton;
            
            if (this.ribbonToggle != null) 
            {
                this.ribbonToggle.ShouldCreatePopup = false;
                this._popup = this.GetTemplateChild(PART_Popup) as System.Windows.Controls.Primitives.Popup;
                this._popup.PlacementTarget = this;
                if (this._popup != null)
                {
                    this.ribbonToggle.Checked += new System.Windows.RoutedEventHandler(this.RibbonToggleButtonChecked);
                    this.ribbonToggle.UnChecked += new System.Windows.RoutedEventHandler(this.RibbonToggleButtonUnChecked);
                }
            }
            
            if (this._popup != null)
            {
                this._popup.Closed += new System.EventHandler(this.PopupClosed);
            }
            Common.PopupService.PopupDismissedEvent += new System.EventHandler(this.DismissPopupEventRaised);
        }

        protected override void OnMouseRightButtonUp(System.Windows.Input.MouseButtonEventArgs e)
        {
            if (Common.PopupService.IsRibbonQAInitialized)
            {
                if (this.ribbonToggle != null)
                {
                    Common.PopupService.EnableRemovalButton();
                }
                else
                {
                    Common.PopupService.DisableRemovalButton();
                }
                Common.PopupService._addItem.IsEnabled = this.IsCloneable;
                Common.PopupService.OpenContextMenu(this);
            }
            base.OnMouseRightButtonUp(e);
            e.Handled = true;
        }
        #endregion

        #region Private Utilities

        private void SetPopupIsOpenProperty(bool value) 
        {
            if (this._popup != null) 
            {
                this._popup.IsOpen = value;
            }
        }

        private bool RemoveGroupFromTab() 
        {
            if (Common.PopupService.RibbonWindow == null) 
            {
                return false;
            }
            if (Common.PopupService.RibbonWindow.RibbonBar.SelectedItem != null) 
            {
                int index = Common.PopupService.RibbonWindow.RibbonBar.SelectedItem.ItemsCollection.IndexOf(this);
                this.Tag = index;
                Common.PopupService.RibbonWindow.RibbonBar.SelectedItem.ItemsCollection.Remove(this);
                Common.PopupService.RibbonWindow.TryMaximizeGroupItems(0);
                return true;
            }
            return false;
        }
        #endregion

        #region Event Handlers
        
        private void DismissPopupEventRaised(object sender, System.EventArgs e) 
        {
            if (sender != this && !(sender is RibbonGroup))
            {
                if (this._togle != null && (bool)this._togle.IsChecked)
                {
                    this._togle.IsChecked = false;
                }
            }
            else if(this.IsDropDownOpen)
            {
                this.IsDropDownOpen = false;
            }
        }
      
        private void launcher_Click(object sender, System.Windows.RoutedEventArgs e) 
        {
            System.Windows.RoutedEventArgs args = new System.Windows.RoutedEventArgs(RibbonGroup.LauncherClickedEvent, this);
            this.RaiseEvent(args);
            Common.PopupService.RaisePopupDismissedEvent(this);
            e.Handled = true;
        }

        private void RibbonToggleButtonChecked(object sender, System.Windows.RoutedEventArgs e) 
        {
            if (!this._popup.IsOpen) 
            {
                this._popup.IsOpen = true;
            }
        }
        
        private void RibbonToggleButtonUnChecked(object sender, System.Windows.RoutedEventArgs e)
        {
            if (this._popup.IsOpen)
            {
                this._popup.IsOpen = false;
            }
        }

        private void PopupClosed(object sender, System.EventArgs e) 
        {
            if (this._togle != null && (bool)this._togle.IsChecked)
            {
                this._togle.IsChecked = false;
            }
            if (this.ribbonToggle != null && (bool)this.ribbonToggle.IsChecked)
            {
                this.ribbonToggle.IsChecked = false;
            }
            if (Common.PopupService.RibbonBar != null &&
                 Common.PopupService.RibbonBar.IsBackgroundMode)
            {
                if (!Common.PopupService.RibbonControlOpening && Common.PopupService.OpeningRibbonControl == this)
                {
                    if (System.Windows.Input.Mouse.Captured != null)
                    {
                        System.Windows.FrameworkElement elt = System.Windows.Input.Mouse.Captured as System.Windows.FrameworkElement;
                        if (elt != null)
                        {
                            if (!(elt is Interfaces.IRibbonControl || elt.TemplatedParent is Interfaces.IRibbonControl || elt.Parent is Interfaces.IRibbonControl))
                            {
                                Common.PopupService.RaisePopupDismissedEvent(this);
                            }
                        }
                    }
                    else 
                    {
                        Common.PopupService.RaisePopupDismissedEvent(this);
                    }
                }
            }
        }
        #endregion

        #region Fields
        private const string _root = "PART_Root";
        private const string PART_Launcher = "PART_Launcher";
        private const string PART_Toogle = "PART_Toogle";
        private const string PART_Popup = "PART_Popup";
        private System.Windows.Controls.Button launcher;
        private Controls.Primitives.ToggleButton _togle;
        private RibbonToggleButton ribbonToggle;
        private System.Windows.Controls.Primitives.Popup _popup;
        private bool isCloneable = false, isClonedItem = false;
        #endregion
        
        #region Events
        public static System.Windows.RoutedEvent LauncherClickedEvent = System.Windows.EventManager.RegisterRoutedEvent
            (
            "LancherClick",
             System.Windows.RoutingStrategy.Bubble,
             typeof(System.Windows.RoutedEventHandler),
             typeof(RibbonGroup)
            );

        [System.ComponentModel.Category("Behavior")]
        public event System.Windows.RoutedEventHandler LauncherClick 
        {
            add { this.AddHandler(RibbonGroup.LauncherClickedEvent, value); }
            remove { this.RemoveHandler(RibbonGroup.LauncherClickedEvent, value); }
        }

        //public event System.Windows.RoutedEventHandler PopupClosed 
        //{
        //    add { this.AddHandler(RibbonGroup.PopupClosedEvent, value); }
        //    remove { this.RemoveHandler(RibbonGroup.PopupClosedEvent, value); }
        //}

        //public static readonly System.Windows.RoutedEvent PopupClosedEvent = System.Windows.EventManager.RegisterRoutedEvent
        //    (
        //    "PopupClosed",
        //     System.Windows.RoutingStrategy.Bubble,
        //     typeof(System.Windows.RoutedEventHandler),
        //     typeof(RibbonGroup)
        //    );
        #endregion
        #region IRibbonControl Implementation
        public Interfaces.IRibbonControl Clone()
        {
            this.Size = RibbonControlSize.Small;
            bool result = this.RemoveGroupFromTab();
            return result ? this : null;
        }
        
        public bool IsLarge
        {
            get { return true; }
        }

        public bool IsClonedItem
        {
            get { return this.isClonedItem; }
        }

        public bool IsCloneable
        {
            get
            {
                return RibbonControlCloningService.CheckIfItemIsCloneable(this);
            }
            internal set { this.isCloneable = value; }
        }


        public void DisposeControl()
        {
            throw new System.NotImplementedException();
        }

        public Interfaces.IRibbonControl ParentControl
        {
            get { return null; }
        }
        #endregion
    }
}
