﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Masterpiece.Controls.Ribbon2010
{
    [System.Windows.TemplatePart(Name=PART_Root, Type=typeof(System.Windows.Shapes.Rectangle))]
    public class RibbonSeparator : System.Windows.Controls.Control, Interfaces.IRibbonControl
    {
        #region Constructors

        static RibbonSeparator() 
        {
            System.Type ownerType=typeof(RibbonSeparator);
            DefaultStyleKeyProperty.OverrideMetadata(ownerType, 
                new System.Windows.FrameworkPropertyMetadata(ownerType));
        }
        #endregion

        #region Properties and Depenency Properties

        public bool Dashed
        {
            get { return (bool)GetValue(DashedProperty); }
            set { SetValue(DashedProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Dashed.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty DashedProperty =
            System.Windows.DependencyProperty.Register("Dashed", typeof(bool), typeof(RibbonSeparator), new System.Windows.PropertyMetadata(false));

        public System.Windows.Controls.Orientation Orientation
        {
            get { return (System.Windows.Controls.Orientation)GetValue(OrientationProperty); }
            set { SetValue(OrientationProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Orientation.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty OrientationProperty =
            System.Windows.DependencyProperty.Register("Orientation", typeof(System.Windows.Controls.Orientation), typeof(RibbonSeparator), new System.Windows.PropertyMetadata(System.Windows.Controls.Orientation.Vertical, OnOrientationPropertyChanged));

        internal double SeparatorWidth
        {
            get { return (double)GetValue(SeparatorWidthProperty); }
            set { SetValue(SeparatorWidthProperty, value); }
        }

        // Using a DependencyProperty as the backing store for SeparatorWidth.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty SeparatorWidthProperty =
            System.Windows.DependencyProperty.Register("SeparatorWidth", typeof(double), typeof(RibbonSeparator));
        
        internal double SeparatorHeight
        {
            get { return (double)GetValue(SeparatorHeightProperty); }
            set { SetValue(SeparatorHeightProperty, value); }
        }

        // Using a DependencyProperty as the backing store for SeparatorHeight.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty SeparatorHeightProperty =
            System.Windows.DependencyProperty.Register("SeparatorHeight", typeof(double), typeof(RibbonSeparator));

        public RibbonControlSize Size
        {
            get { return (RibbonControlSize)GetValue(SizeProperty); }
            set { SetValue(SizeProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Size.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty SizeProperty =
            System.Windows.DependencyProperty.Register("Size", typeof(RibbonControlSize), typeof(RibbonSeparator), new System.Windows.PropertyMetadata(RibbonControlSize.Small));

        public System.Windows.Media.Brush Stroke
        {
            get { return (System.Windows.Media.Brush)GetValue(StrokeProperty); }
            set { SetValue(StrokeProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Stroke.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty StrokeProperty =
            System.Windows.DependencyProperty.Register("Stroke", typeof(System.Windows.Media.Brush), typeof(RibbonSeparator), new System.Windows.PropertyMetadata(null));

        
        #endregion

        #region Depenency Property Changed Callbacks

        private static void OnSizePropertyChanged(System.Windows.DependencyObject o, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
            RibbonControlSize size = (RibbonControlSize)e.NewValue;
            RibbonSeparator separator = o as RibbonSeparator;
            if (size == RibbonControlSize.Large)
            {
                if (separator.Orientation == System.Windows.Controls.Orientation.Vertical)
                {
                    separator.SetValue(RibbonSeparator.SeparatorWidthProperty, 0.8D);
                    separator.SetValue(RibbonSeparator.SeparatorHeightProperty, 68.0D);
                }
                else
                {
                    separator.SetValue(RibbonSeparator.SeparatorWidthProperty, 68.0D);
                    separator.SetValue(RibbonSeparator.SeparatorHeightProperty, 0.8D);
                }
            }
            else 
            {
                if (separator.Orientation == System.Windows.Controls.Orientation.Vertical)
                {
                    separator.SetValue(RibbonSeparator.SeparatorWidthProperty, 0.8D);
                    separator.SetValue(RibbonSeparator.SeparatorHeightProperty, 23.0D);
                }
                else
                {
                    separator.SetValue(RibbonSeparator.SeparatorWidthProperty, 23.0D);
                    separator.SetValue(RibbonSeparator.SeparatorHeightProperty, 0.8D);
                }
            }
        }

        private static void OnOrientationPropertyChanged(System.Windows.DependencyObject o, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
        }

        #endregion

        #region Overrides

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();

            this._root = this.GetTemplateChild(PART_Root) as System.Windows.Shapes.Rectangle;
            if (this.Dashed)
            {
                this._root.StrokeDashArray.Add(2.0D);
                this._root.StrokeDashArray.Add(2.0D);
            }
        }
        protected override void OnInitialized(EventArgs e)
        {
            base.OnInitialized(e);

            
        }
        #endregion

        #region Private Utilities
        #endregion

        #region Event Handlers
        #endregion

        #region Fields
        internal System.Windows.Shapes.Rectangle _root;
        private const string PART_Root="PART_Root";
        #endregion
        #region IRibbonControl Implementation
        public bool IsLarge
        {
            get { return this.Size == RibbonControlSize.Large; }
        }

        public Interfaces.IRibbonControl Clone()
        {
            throw new NotImplementedException();
        }


        public bool IsClonedItem
        {
            get { return false; }
        }

        public bool IsCloneable
        {
            get { return false; }
        }


        public void DisposeControl()
        {
            throw new NotImplementedException();
        }

        public Interfaces.IRibbonControl ParentControl
        {
            get { return null; }
        }
        #endregion
    }
}
