﻿

namespace Masterpiece.Controls.Ribbon2010
{
    public partial class RibbonStackPanel : System.Windows.Controls.Panel, System.Windows.Controls.Primitives.IScrollInfo, Interfaces.IRibbonControl
    {
        #region Constructors
        public RibbonStackPanel() 
        {
            this._translate = new System.Windows.Media.TranslateTransform();
            this.RenderTransform = this._translate;
        }
        #endregion

        #region Properties and Depenency Properties

        [System.ComponentModel.Bindable(true), 
        System.ComponentModel.Browsable(true), 
        System.ComponentModel.Category("Behavior"),
        System.ComponentModel.ReadOnly(false)]
        public bool AnimatedScroll
        {
            get { return (bool)GetValue(AnimatedScrollProperty); }
            set { SetValue(AnimatedScrollProperty, value); }
        }

        // Using a DependencyProperty as the backing store for AnimatedScroll.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty AnimatedScrollProperty =
            System.Windows.DependencyProperty.Register("AnimatedScroll", typeof(bool), typeof(RibbonStackPanel), new System.Windows.PropertyMetadata(true));

        [System.ComponentModel.Bindable(true),
        System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Behavior"),
        System.ComponentModel.ReadOnly(false)]
        public double ScrollBy
        {
            get { return (double)GetValue(ScrollByProperty); }
            set { SetValue(ScrollByProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ScrollBy.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty ScrollByProperty =
            System.Windows.DependencyProperty.Register("ScrollBy", typeof(double), typeof(RibbonStackPanel), new System.Windows.PropertyMetadata(10.0D));


        [System.ComponentModel.Bindable(true),
        System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Behavior"),
        System.ComponentModel.ReadOnly(false)]
        public System.Windows.Controls.Orientation Orientation
        {
            get { return (System.Windows.Controls.Orientation)GetValue(OrientationProperty); }
            set { SetValue(OrientationProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Orientation.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty OrientationProperty =
            System.Windows.DependencyProperty.Register("Orientation", typeof(System.Windows.Controls.Orientation), typeof(RibbonStackPanel), new System.Windows.PropertyMetadata(System.Windows.Controls.Orientation.Vertical));
        
        [System.ComponentModel.Bindable(true),
        System.ComponentModel.Browsable(true),
        System.ComponentModel.Category("Behavior"),
        System.ComponentModel.ReadOnly(false)]
        public bool FitsToViewport
        {
            get { return (bool)GetValue(FitsToViewportProperty); }
            set { SetValue(FitsToViewportProperty, value); }
        }

        // Using a DependencyProperty as the backing store for FitsToViewport.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty FitsToViewportProperty =
            System.Windows.DependencyProperty.Register("FitsToViewport", typeof(bool), typeof(RibbonStackPanel), new System.Windows.PropertyMetadata(false));


        public bool Measured
        {
            get { return (bool)GetValue(MeasuredProperty); }
        }

        // Using a DependencyProperty as the backing store for Measured.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyPropertyKey MeasuredPropertyKey =
            System.Windows.DependencyProperty.RegisterReadOnly("Measured", typeof(bool), typeof(RibbonStackPanel), new System.Windows.PropertyMetadata(false));
        internal static readonly System.Windows.DependencyProperty MeasuredProperty = MeasuredPropertyKey.DependencyProperty;

        internal int Interval
        {
            get { return (int)GetValue(IntervalProperty); }
            set { SetValue(IntervalProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Interval.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty IntervalProperty =
            System.Windows.DependencyProperty.Register("Interval", typeof(int), typeof(RibbonStackPanel), new System.Windows.PropertyMetadata(1));

        #endregion

        #region Depenency Property Changed Callbacks

        protected override void OnInitialized(System.EventArgs e)
        {
            base.OnInitialized(e);
        }
        #endregion

        #region Overrides

        protected override System.Windows.Size MeasureOverride(System.Windows.Size availableSize)
        {
            
           double w = 0;
            System.Windows.Size sizeInfinite = new System.Windows.Size(double.PositiveInfinity, double.PositiveInfinity);
            System.Windows.Size size = new System.Windows.Size();
            if (this.Orientation == System.Windows.Controls.Orientation.Horizontal)
            {
                foreach (System.Windows.UIElement item in this.InternalChildren)
                {
                    item.Measure(sizeInfinite);
                    if (item is RibbonSeparator)
                    {
                        w += 1.8D;
                    }
                    else
                    {
                        w += item.DesiredSize.Width + this.Interval;
                    }
                }
                if (availableSize.Width == double.PositiveInfinity)
                {
                    size.Width = w;
                }
                else
                {
                    if (!this.FitsToViewport)
                    {
                        size.Width = System.Math.Min(availableSize.Width, w);
                    }
                    else 
                    {
                        if (availableSize.Width < w)
                        {
                            this.MinimizeItems(availableSize, w);
                        }
                        size.Width = availableSize.Width;
                    }
                }
                
                size.Height = this.GetMaxHeight();


            }
            else 
            {
                foreach (System.Windows.UIElement item in this.InternalChildren)
                {
                    item.Measure(sizeInfinite);
                    if (item is RibbonSeparator)
                    {
                        w += 1.8D;
                    }
                    else
                    {
                        w += item.DesiredSize.Height + this.Interval;
                    }
                }
                if (availableSize.Height == double.PositiveInfinity)
                {
                    size.Height = w;
                }
                else
                {
                    size.Height = System.Math.Min(availableSize.Height, w);
                }
                size.Width = this.GetMaxWidth();
            }
            if (this.ScrollOwner != null)
            {
                this.UpdateScrollInfo(size);
            }
            return size;

        }

        protected override System.Windows.Size ArrangeOverride(System.Windows.Size finalSize)
        {
            double x = 0;
            if (this.Orientation == System.Windows.Controls.Orientation.Horizontal)
            {
                foreach (System.Windows.UIElement item in this.InternalChildren)
                {
                    if (item is RibbonSeparator)
                    {
                        RibbonSeparator separator = item as RibbonSeparator;
                        separator.SetValue(RibbonSeparator.SeparatorWidthProperty, 0.8);
                        separator.SetValue(RibbonSeparator.SeparatorHeightProperty, finalSize.Height < 10 ? 5 : finalSize.Height - 5);
                        separator.Arrange(new System.Windows.Rect(x, 2.5, 0.8D, finalSize.Height < 10 ? 5 : finalSize.Height - 5));
                        x += 1.8D;
                    }
                    else
                    {
                        item.Arrange(new System.Windows.Rect(x, 0, item.DesiredSize.Width, finalSize.Height));
                        x += item.DesiredSize.Width + this.Interval;
                    }
                }
            }
            else
            {
                foreach (System.Windows.UIElement item in this.InternalChildren)
                {
                    if (item is RibbonSeparator)
                    {
                        RibbonSeparator separator = item as RibbonSeparator;
                        separator.SetValue(RibbonSeparator.SeparatorWidthProperty, finalSize.Width);
                        separator.SetValue(RibbonSeparator.SeparatorHeightProperty, 0.8);
                        separator.Arrange(new System.Windows.Rect(0, x, finalSize.Width, 0.8D));
                        x += 1.8D;
                        
                    }
                    else
                    {
                        item.Arrange(new System.Windows.Rect(0, x, finalSize.Width, item.DesiredSize.Height));
                        x += item.DesiredSize.Height + this.Interval;
                    }
                }
            }
            this.SetValue(RibbonStackPanel.MeasuredPropertyKey, true);
            return finalSize;
        }
        #endregion

        #region Private Utilities

        private double GetMaxHeight() 
        {
            double max = 0;
            foreach (System.Windows.UIElement item in this.InternalChildren)
            {
                max = System.Math.Max(item.DesiredSize.Height, max);
                
            }
            return max;
        }

        private double GetMaxWidth() 
        {
            double max = 0;
            foreach (System.Windows.UIElement item in this.InternalChildren)
            {
                max = System.Math.Max(item.DesiredSize.Width, max);
                
            }
            return max;
        }

        private void ArrangeSeparator(RibbonSeparator separator, double x, System.Windows.Controls.Orientation orinentation)
        {
            double max = 0;
            if (orinentation == System.Windows.Controls.Orientation.Horizontal)
            {
                max = this.GetMaxWidth();
                separator.SetValue(RibbonSeparator.SeparatorWidthProperty, max-10);
                separator.SetValue(RibbonSeparator.SeparatorHeightProperty, 0.8);
                separator.Arrange(new System.Windows.Rect(0, x, separator.SeparatorWidth, separator.SeparatorHeight));
            }
            else
            {
                max = this.GetMaxHeight();
                separator.SetValue(RibbonSeparator.SeparatorWidthProperty, 0.8D);
                separator.SetValue(RibbonSeparator.SeparatorHeightProperty, max-15);
                separator.Arrange(new System.Windows.Rect(x, 0, separator.SeparatorWidth, separator.SeparatorHeight));
            }

        }

        private void MinimizeItems(System.Windows.Size avaliableSize, double actualWidth)
        {
            int index = this.InternalChildren.Count - 1; double current = 0;
            System.Windows.Size size = new System.Windows.Size(double.PositiveInfinity, double.PositiveInfinity);
            while (actualWidth > avaliableSize.Width && index >= 0)
            {
                System.Windows.UIElement e = this.InternalChildren[index];

                if (e is RibbonGroup)
                {
                    RibbonGroup group = e as RibbonGroup;

                    if (!group.Minimized)
                    {
                        if (group.CanMinimize)
                        {
                            actualWidth -= group.DesiredSize.Width;
                            current = group.DesiredSize.Width;
                            group.SetValue(RibbonGroup.MinimizedPropertyKey, true);
                            group.Measure(size);
                            if (System.Math.Abs(current - group.DesiredSize.Width) <= 25)
                            {
                                actualWidth += current;
                                group.SetValue(RibbonGroup.CanMinimizePropertyKey, false);
                                continue;
                            }
                            else 
                            {
                                actualWidth += group.DesiredSize.Width;
                            }
                        }
                    }
                }
                index--;
            }

        }

        private void MaximizeObjects(System.Windows.Size avaliableSize, double actualWidth) 
        {
            int index = 0; double current = 0;
            while (actualWidth < avaliableSize.Width && index <= this.InternalChildren.Count-1)
            {
                System.Windows.UIElement e = this.InternalChildren[index];

                if (e is RibbonGroup)
                {
                    RibbonGroup group = e as RibbonGroup;

                    if (group.Minimized)
                    {
                        actualWidth -= group.DesiredSize.Width;
                        current = group.DesiredSize.Width;
                        group.SetValue(RibbonGroup.MinimizedPropertyKey, false);
                        group.Measure(new System.Windows.Size(double.PositiveInfinity, double.PositiveInfinity));
                        actualWidth += group.DesiredSize.Width;
                    }
                }
                index++;

            }

        }
        #endregion

        #region Event Handlers
        private  void ScrollOwner_SizeChanged(object sender, System.Windows.SizeChangedEventArgs e) 
        {
            if (e.WidthChanged) 
            {
                if (this._extent.Width < this._viewport.Width)
                {
                    this.MaximizeObjects(this._viewport, this._extent.Width);
                }
                
            }
        }
        #endregion

        #region Fields
        internal System.Windows.Media.TranslateTransform _translate;
        private System.Windows.Size _extent=new System.Windows.Size(0, 0), _viewport=new System.Windows.Size(0, 0);
        internal System.Windows.Point offset = new System.Windows.Point(0, 0);
        private System.Windows.Controls.ScrollViewer scrollViewer;
        private bool canHScroll=false, canVScroll=false;
        #endregion

        #region IRibbonControl Implementation

        public Interfaces.IRibbonControl Clone()
        {
            throw new System.NotImplementedException();
        }


        public bool IsLarge
        {
            get { return true; }
        }


        public bool IsClonedItem
        {
            get { return false; }
        }

        public bool IsCloneable
        {
            get { return false; }
        }


        public void DisposeControl()
        {
            throw new System.NotImplementedException();
        }

        public Interfaces.IRibbonControl ParentControl
        {
            get { return null; }
        }
        #endregion
    }
}
