﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Masterpiece.Controls.Ribbon2010
{
    public sealed class GroupCollection:System.Collections.ObjectModel.ObservableCollection<RibbonGroup> { }
    [System.Windows.Markup.ContentProperty("ItemsCollection")]
    public class RibbonTabItem : System.Windows.Controls.Control,Interfaces.IRibbonControl
    {
        #region Constructors
        static RibbonTabItem()
        {
            DefaultStyleKeyProperty.OverrideMetadata(typeof(RibbonTabItem),
                new System.Windows.FrameworkPropertyMetadata(typeof(RibbonTabItem)));

        }
        public RibbonTabItem()
            : base()
        {
            this.ItemsCollection = new GroupCollection();
        }
        #endregion

        #region Properties and Depenency Properties

        public GroupCollection ItemsCollection
        {
            get { return (GroupCollection)GetValue(ItemsCollectionProperty); }
            set { SetValue(ItemsCollectionProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ItemsCollection.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty ItemsCollectionProperty =
            System.Windows.DependencyProperty.Register("ItemsCollection", typeof(GroupCollection), typeof(RibbonTabItem));

               

        public bool IsSelected
        {
            get { return (bool)GetValue(IsSelectedProperty); }
            set { SetValue(IsSelectedProperty, value); }

        }

        // Using a DependencyProperty as the backing store for IsSelected.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty IsSelectedProperty =
            System.Windows.DependencyProperty.Register("IsSelected", typeof(bool), typeof(RibbonTabItem), new System.Windows.PropertyMetadata(false, OnIsSelectedChanged));

        public string Header
        {
            get { return (string)GetValue(HeaderProperty); }
            set { SetValue(HeaderProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Header.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty HeaderProperty =
            System.Windows.DependencyProperty.Register("Header", typeof(string), typeof(RibbonTabItem), new System.Windows.PropertyMetadata(""));

        internal bool IsRibbonBarOnBackgroundMode
        {
            get { return (bool)GetValue(IsRibbonBarOnBackgroundModeProperty); }
            set { SetValue(IsRibbonBarOnBackgroundModeProperty, value); }
        }

        // Using a DependencyProperty as the backing store for IsRibbonBarOnBackgroundMode.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty IsRibbonBarOnBackgroundModeProperty =
            System.Windows.DependencyProperty.Register("IsRibbonBarOnBackgroundMode", typeof(bool), typeof(RibbonTabItem), new System.Windows.FrameworkPropertyMetadata(false));

        #endregion

        #region Depenency Property Changed Callbacks

        private static void OnIsSelectedChanged(System.Windows.DependencyObject o, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
            if ((bool)e.NewValue) 
            {
                RibbonTabItem item = o as RibbonTabItem;
                RibbonBar bar = item.Parent as RibbonBar;
                if (bar != null) 
                {
                    bar.ClearValue(RibbonBar.SelectedItemProperty);
                    bar.SetValue(RibbonBar.SelectedItemProperty, item);
                    bar.SetValue(RibbonBar.SelectedIndexProperty, bar.RibbonTabItemsCollection.IndexOf(item));
                    if (!bar.IsExpanded)
                    {
                        bar.AnimateGridLength();
                    }
                }
                
            }
        }

        #endregion

        #region Overrieds
        protected override void OnMouseLeftButtonDown(System.Windows.Input.MouseButtonEventArgs e)
        {
            base.OnMouseLeftButtonDown(e);
            RibbonBar bar = null;
            if (this.VisualParent != null)
            {
                System.Windows.Controls.Panel visualParent = this.VisualParent as System.Windows.Controls.Panel;
                if (visualParent != null)
                {
                    bar = visualParent.TemplatedParent as RibbonBar;
                    if (bar == null)
                    {
                        return;
                    }
                }
            }
            if (e.ClickCount == 1)
            {
                if (e.LeftButton == System.Windows.Input.MouseButtonState.Pressed)
                {
                    if (bar.ApplicationMenu != null)
                    {
                        bar.ApplicationMenu._root.IsChecked = false;
                    }
                    RibbonTabItem tab = bar.GetValue(RibbonBar.SelectedItemProperty) as RibbonTabItem;
                    bool shouldCloseDropDown = tab != null && tab == this;
                    if (tab != this)
                    {
                        this.SetValue(RibbonTabItem.IsSelectedProperty, true);
                        bar.ClearValue(RibbonBar.SelectedItemProperty);
                        bar.SetValue(RibbonBar.SelectedItemProperty, this);
                        bar.SetValue(RibbonBar.SelectedIndexProperty, bar.RibbonTabItemsCollection.IndexOf(this));
                        if (tab != null)
                        {
                            tab.SetValue(RibbonTabItem.IsSelectedProperty, false);
                        }
                        
                    }
                    this.OperateRibbonBar(bar, shouldCloseDropDown);
                }
            }
            else if (e.ClickCount == 2) 
            {
                if (bar != null)
                {
                    bar.IsBackgroundMode = !bar.IsBackgroundMode;
                }
            }
           
        }

        #endregion

        #region Private Utilities

        private void OperateRibbonBar(RibbonBar bar, bool shouldCloseDropDown) 
        {
            if (!bar.IsBackgroundMode)
            {
                if (!bar.IsExpanded)
                {
                    bar.AnimateGridLength();
                }
            }
            else
            {
                if (shouldCloseDropDown)
                {
                    Common.PopupService.RaisePopupDismissedEvent(bar);
                }
                else
                {
                    bar.OpenDropDown();
                }
            }
        }
        #endregion

        #region Event Handlers
        #endregion
        #region IRibbonControl Implementation
        public Interfaces.IRibbonControl Clone()
        {
            throw new NotImplementedException();
        }

        public bool IsLarge
        {
            get { return true; }
        }


        public bool IsClonedItem
        {
            get { return false; }
        }

        public bool IsCloneable
        {
            get { return false; }
        }


        public void DisposeControl()
        {
            throw new NotImplementedException();
        }

        public Interfaces.IRibbonControl ParentControl
        {
            get { return null; }
        }
        #endregion
    }
}
