﻿namespace Masterpiece.Controls.Ribbon2010
{
    [System.ComponentModel.DefaultBindingProperty("IsChecked")]
    [System.ComponentModel.DefaultProperty("IsChecked")]
    [System.ComponentModel.DefaultEvent("Checked")]
    [System.Windows.Markup.ContentProperty("RibbonPopupMenu")]
    public class RibbonToggleButton : Primitives.RibbonButtonBase, Interfaces.IRibbonControl
    {
        #region Constructors

        static RibbonToggleButton() 
        {
            System.Type ownerType=typeof(RibbonToggleButton);
            DefaultStyleKeyProperty.OverrideMetadata(ownerType,
                new System.Windows.FrameworkPropertyMetadata(ownerType));
        }
        public RibbonToggleButton():
            base() 
        {
        }
        #endregion
        
        #region Properties and Depenency Properties

        [System.Windows.Localizability(System.Windows.LocalizationCategory.None, Readability = System.Windows.Readability.Unreadable)]
        [System.ComponentModel.Bindable(true), System.ComponentModel.Browsable(false), System.ComponentModel.ReadOnly(false)]
        internal bool ShouldCreatePopup
        {
            get { return (bool)GetValue(ShouldCreatePopupProperty); }
            set { SetValue(ShouldCreatePopupProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ShouldCreatePopup.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty ShouldCreatePopupProperty =
            System.Windows.DependencyProperty.Register("ShouldCreatePopup", typeof(bool), typeof(RibbonToggleButton), new System.Windows.FrameworkPropertyMetadata(true, new System.Windows.PropertyChangedCallback(RibbonToggleButton.OnShouldCreatePopupPropertyChanged)));

        [System.ComponentModel.Category("Appearance")]
        [System.ComponentModel.TypeConverter(typeof(System.Windows.NullableBoolConverter))]
        [System.Windows.Localizability(System.Windows.LocalizationCategory.None, 
                                       Readability = System.Windows.Readability.Unreadable)]
        [System.ComponentModel.Bindable(true), 
        System.ComponentModel.Browsable(false), 
        System.ComponentModel.ReadOnly(false)]
        public bool? IsChecked
        {
            get
            {
                object value = this.GetValue(IsCheckedProperty);
                if (value == null)
                {
                    return new System.Nullable<bool>();
                }
                else
                {
                    return new System.Nullable<bool>((bool)value);
                }
            }
            set { SetValue(IsCheckedProperty, value.HasValue ? (bool?)value.Value : null); }
        }

        // Using a DependencyProperty as the backing store for IsChecked.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty IsCheckedProperty =
            System.Windows.DependencyProperty.Register("IsChecked", typeof(bool?), typeof(RibbonToggleButton), new System.Windows.FrameworkPropertyMetadata
                (
                false,
                System.Windows.FrameworkPropertyMetadataOptions.BindsTwoWayByDefault
                | System.Windows.FrameworkPropertyMetadataOptions.Journal,
                new System.Windows.PropertyChangedCallback(OnIsCheckedPropertyChanged)));

        public RibbonPopupMenu RibbonPopupMenu
        {
            get { return (RibbonPopupMenu)GetValue(RibbonPopupMenuProperty); }
            set { SetValue(RibbonPopupMenuProperty, value); }
        }

        // Using a DependencyProperty as the backing store for RibbonPopupMenu.  This enables animation, styling, binding, etc...
        public static readonly System.Windows.DependencyProperty RibbonPopupMenuProperty =
            System.Windows.DependencyProperty.Register("RibbonPopupMenu", typeof(RibbonPopupMenu), typeof(RibbonToggleButton), new System.Windows.PropertyMetadata((RibbonPopupMenu)null, new System.Windows.PropertyChangedCallback(RibbonToggleButton.OnRibbonPopupMenuPropertyChanged)));

        internal bool HasContentInternal
        {
            get { return (bool)GetValue(HasContentInternalProperty); }
            set { SetValue(HasContentInternalProperty, value); }
        }

        // Using a DependencyProperty as the backing store for HasContentInternal.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty HasContentInternalProperty =
            System.Windows.DependencyProperty.Register("HasContentInternal", typeof(bool), typeof(RibbonToggleButton), new System.Windows.PropertyMetadata(false));


        public bool IsDropDownOpen
        {
            get { return (bool)GetValue(IsDropDownOpenProperty); }
            set { SetValue(IsDropDownOpenProperty, value); }
        }
        public static readonly System.Windows.DependencyProperty IsDropDownOpenProperty = RibbonComboBox.IsDropDownOpenProperty.AddOwner(typeof(RibbonToggleButton), new System.Windows.PropertyMetadata(
            false, 
            new System.Windows.PropertyChangedCallback(RibbonToggleButton.OnIsDropDownOpenPropertyChanged),
            new System.Windows.CoerceValueCallback(RibbonToggleButton.CoerceIsDropDownOpenProperty)));

        #endregion

        #region Depenency Property Changed Callbacks

        private static System.Object CoerceIsDropDownOpenProperty(System.Windows.DependencyObject d, System.Object value) 
        {
            bool newValue = (bool)value;
            RibbonToggleButton toggleButton = (RibbonToggleButton)d;
            if (newValue)
            {
                Common.PopupService.RibbonControlOpening = true;
                Common.PopupService.OpeningRibbonControl = toggleButton;
            }
            return value;
        }
        private static void OnIsDropDownOpenPropertyChanged(System.Windows.DependencyObject o, System.Windows.DependencyPropertyChangedEventArgs e)
        {
            bool newValue = (bool)e.NewValue, oldValue = !newValue;
            RibbonToggleButton toggleButton = (RibbonToggleButton)o;
            System.Windows.Data.BindingExpression expression = toggleButton.GetBindingExpression(RibbonToggleButton.IsDropDownOpenProperty);
            if (!newValue)
            {
                if ((bool)toggleButton.IsChecked)
                {
                    toggleButton.IsChecked = false;
                }
                if (toggleButton.ClonedChild != null && (bool)toggleButton.ClonedChild.IsChecked) 
                {
                    toggleButton.ClonedChild.IsChecked = false;
                }
            }
            toggleButton.SetPopupIsOpenValue(newValue);
        }

        private static void OnRibbonPopupMenuPropertyChanged(System.Windows.DependencyObject o, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
            RibbonToggleButton btn = (RibbonToggleButton)o;
            RibbonPopupMenu newValue = (RibbonPopupMenu)e.NewValue, oldValue = (RibbonPopupMenu)e.OldValue;
            System.Windows.Data.BindingExpression bindingExpression = null;
            if (oldValue != null)
            {
                btn.ClearBindings(oldValue);
            }
            if (newValue != null)
            {
                btn.SetValue(RibbonToggleButton.HasContentInternalProperty, true);
                if (btn.Popup == null)
                {
                    btn.Popup = Common.PopupService.PopupBuilder(newValue as RibbonPopupMenu);
                    btn.Popup.Closed += new System.EventHandler(btn.PopupDismissed);
                    btn.Popup.PlacementTarget = btn;
                    btn.SetBindings();
                }
                else 
                {
                    Masterpiece.Controls.DropShadowChrome shadowChrome = btn.Popup.Child as Masterpiece.Controls.DropShadowChrome;
                    shadowChrome.Content = e.NewValue;
                    btn.SetBindings();
                }
                if (btn.ClonedChild != null) 
                {
                    bindingExpression = btn.ClonedChild.GetBindingExpression(RibbonToggleButton.IsDropDownOpenProperty);
                    if (bindingExpression == null)
                    {
                        System.Windows.Data.BindingOperations.SetBinding
                         (
                            btn.ClonedChild,
                            RibbonToggleButton.IsDropDownOpenProperty,
                            new System.Windows.Data.Binding("IsChecked") { Source = btn.ClonedChild, Mode = System.Windows.Data.BindingMode.TwoWay }
                         );
                    }
                }
            }
            else 
            {
                btn.SetValue(RibbonToggleButton.HasContentInternalProperty, false);
                if (btn.ClonedChild != null)
                {
                    bindingExpression = btn.ClonedChild.GetBindingExpression(RibbonToggleButton.IsCheckedProperty);
                    if (bindingExpression == null)
                    {
                        System.Windows.Data.BindingOperations.SetBinding
                               (
                                  btn.ClonedChild,
                                  RibbonToggleButton.IsCheckedProperty,
                                  new System.Windows.Data.Binding("IsChecked") { Source = btn, Mode = System.Windows.Data.BindingMode.TwoWay }
                               );
                    }
                    bindingExpression = btn.ClonedChild.GetBindingExpression(RibbonToggleButton.IsDropDownOpenProperty);
                    if (bindingExpression != null) 
                    {
                        System.Windows.Data.BindingOperations.ClearBinding(btn.ClonedChild, RibbonToggleButton.IsDropDownOpenProperty);
                    }
                }
            }
        }

        
        private static void OnShouldCreatePopupPropertyChanged(System.Windows.DependencyObject o, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
            bool oldValue = (bool)e.OldValue, newValue = (bool)e.NewValue;
            RibbonToggleButton btn = (RibbonToggleButton)o;
            if (oldValue && !newValue) 
            {
                if (btn.Popup != null) 
                {
                    btn.Popup = null;
                }
            }
        }

        private static void OnIsCheckedPropertyChanged(System.Windows.DependencyObject o, System.Windows.DependencyPropertyChangedEventArgs e)
        {
            RibbonToggleButton btn = (RibbonToggleButton)o;
            bool? newValue = (bool?)e.NewValue;
            bool? oldValue = (bool?)e.OldValue;
            if (newValue == true)
            {
                btn.OnChecked(new System.Windows.RoutedEventArgs(RibbonToggleButton.CheckedEvent));
            }
            else if (newValue == false)
            {
                btn.OnUnchecked(new System.Windows.RoutedEventArgs(RibbonToggleButton.UnCheckedEvent));
            }
            else 
            {
                btn.ClearValue(RibbonToggleButton.IsCheckedProperty);
            }
        }

        #endregion

        #region Overrides

        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            Common.PopupService.PopupDismissedEvent -= this.PopupDismissed;
            Common.PopupService.PopupDismissedEvent += new System.EventHandler(this.PopupDismissed);
        }
        protected virtual void OnChecked(System.Windows.RoutedEventArgs e)
        {
            this.RaiseEvent(e);
        }

        protected virtual void OnUnchecked(System.Windows.RoutedEventArgs e)
        {
            this.RaiseEvent(e);
        }

        protected override void OnClick()
        {
            this.OnToggle();
            base.OnClick();
            if (!this.HasContentInternal)
            {
                Common.PopupService.RaisePopupDismissedEvent(this);
            }
        }

        protected override void OnMouseRightButtonUp(System.Windows.Input.MouseButtonEventArgs e)
        {
            if (Common.PopupService.IsRibbonQAInitialized)
            {
                if (this.TemplatedParent != null && this.TemplatedParent is RibbonGroup)
                {
                    RibbonGroup group = this.TemplatedParent as RibbonGroup;
                    if (group.Size == RibbonControlSize.Small)
                    {
                        Common.PopupService.EnableRemovalButton();
                        Common.PopupService.OpenContextMenu(group);
                        return;
                    }
                }
                if (this.ParentUI != null)
                {
                    Common.PopupService.EnableRemovalButton();
                }
                else
                {
                    Common.PopupService.DisableRemovalButton();
                }
                Common.PopupService._addItem.IsEnabled = this.IsCloneable;
                Common.PopupService.OpenContextMenu(this);
            }
            base.OnMouseRightButtonUp(e);
            e.Handled = true;
        }
       
        #endregion

        #region Private Utilities

        private void SetBindings() 
        {
            this.RibbonPopupMenu.Tag = this;
            System.Windows.Data.BindingOperations.SetBinding
                     (
                         this.RibbonPopupMenu,
                         RibbonPopupMenu.IsDropDownOpenProperty,
                         new System.Windows.Data.Binding("IsDropDownOpen") { Source = this, Mode = System.Windows.Data.BindingMode.TwoWay }
                     );
            System.Windows.Data.BindingOperations.SetBinding
                (
                    this,
                    RibbonToggleButton.IsCheckedProperty,
                    new System.Windows.Data.Binding("IsDropDownOpen") { Source = this, Mode = System.Windows.Data.BindingMode.TwoWay }
                );
        }

        private void SetBindings(RibbonToggleButton clonedItem) 
        {
            if (this.SmallImage != null)
            {
                System.Windows.Data.BindingOperations.SetBinding
                          (
                             clonedItem,
                             RibbonToggleButton.SmallImageProperty,
                             new System.Windows.Data.Binding("SmallImage") { Source = this }
                          );
            }
            else if (this.LargeImage != null)
            {
                System.Windows.Data.BindingOperations.SetBinding
                          (
                             clonedItem,
                             RibbonToggleButton.SmallImageProperty,
                             new System.Windows.Data.Binding("LargeImage") { Source = this }
                          );
            }
            if (!this.HasContentInternal)
            {
                System.Windows.Data.BindingOperations.SetBinding
                                (
                                   clonedItem,
                                   RibbonToggleButton.IsCheckedProperty,
                                   new System.Windows.Data.Binding("IsChecked") { Source = this }
                                );
            }
            else 
            {
                System.Windows.Data.BindingOperations.SetBinding
                         (
                            clonedItem,
                            RibbonToggleButton.IsDropDownOpenProperty,
                            new System.Windows.Data.Binding("IsChecked") { Source = clonedItem, Mode = System.Windows.Data.BindingMode.TwoWay }
                         );
            }
            System.Windows.Data.BindingOperations.SetBinding
                         (
                            clonedItem,
                            RibbonToggleButton.HasContentInternalProperty,
                            new System.Windows.Data.Binding("HasContentInternal") { Source = this, Mode = System.Windows.Data.BindingMode.TwoWay }
                         );
            
        }
        private void ClearBindings(RibbonPopupMenu menu)
        {
            if (menu == null) 
            {
                return;
            }

            System.Windows.Data.BindingExpression expression = menu.GetBindingExpression(RibbonPopupMenu.IsDropDownOpenProperty);
            if (expression != null && expression.DataItem == this)
            {
                System.Windows.Data.BindingOperations.ClearBinding(menu, RibbonPopupMenu.IsDropDownOpenProperty);
            }
            expression = this.GetBindingExpression(RibbonToggleButton.IsCheckedProperty);
            if (expression != null && expression.DataItem == this)
            {
                System.Windows.Data.BindingOperations.ClearBinding(this, RibbonToggleButton.IsCheckedProperty);
            }
        }
        
        private void OnToggle()
        {
            if (this.IsChecked == true)
            {
                this.IsChecked = (bool?)false;
            }
            else
            {
                this.IsChecked = this.IsChecked.HasValue;
            }
        }

        private void CloseDropDown() 
        {
            if (this.IsDropDownOpen) 
            {
                this.IsDropDownOpen = false;
            }
            else if (this.ClonedChild != null && this.ClonedChild.IsDropDownOpen) 
            {
                this.ClonedChild.IsDropDownOpen = false;
            }
        }

        private void SetPopupIsOpenValue(bool value)
        {
            if (this.IsClonedItem)
            {
                if (this.ParentUI != null && this.ParentUI.Popup != null)
                {
                    if (value)
                    {
                        this.ParentUI.Popup.PlacementTarget = this;
                    }
                    else 
                    {
                        this.ParentUI.Popup.ClearValue(System.Windows.Controls.Primitives.Popup.PlacementTargetProperty);
                    }
                    this.ParentUI.Popup.IsOpen = value;
                }
                return;
            }
            
            else if (this.Popup != null)
            {
                if (value)
                {
                    this.Popup.PlacementTarget = this;
                }
                this.Popup.IsOpen = value;
            }
        }
        #endregion

        #region Event Handlers

        private void PopupDismissed(object sender, System.EventArgs e) 
        {
            this.CloseDropDown();
        }

        [System.ComponentModel.Category("Behavior")]
        public event System.Windows.RoutedEventHandler Checked 
        {
            add { this.AddHandler(Ribbon2010.RibbonToggleButton.CheckedEvent, value); }
            remove { this.RemoveHandler(Ribbon2010.RibbonToggleButton.CheckedEvent, value); }
        }
        
        [System.ComponentModel.Category("Behavior")]
        public event System.Windows.RoutedEventHandler UnChecked
        {
            add { this.AddHandler(Ribbon2010.RibbonToggleButton.UnCheckedEvent, value); }
            remove { this.RemoveHandler(Ribbon2010.RibbonToggleButton.UnCheckedEvent, value); }
        }

        public static readonly System.Windows.RoutedEvent CheckedEvent = System.Windows.EventManager.RegisterRoutedEvent
            (
            "Checked", 
            System.Windows.RoutingStrategy.Bubble,
            typeof(System.Windows.RoutedEventHandler), 
            typeof(RibbonToggleButton)
            );
        public static readonly System.Windows.RoutedEvent UnCheckedEvent = System.Windows.EventManager.RegisterRoutedEvent
            (
            "UnChecked",
            System.Windows.RoutingStrategy.Bubble,
            typeof(System.Windows.RoutedEventHandler),
            typeof(Masterpiece.Controls.Ribbon2010.RibbonToggleButton)
            );
        #endregion

        #region Fields
        
        internal System.Windows.Controls.Primitives.Popup Popup;
        
        internal RibbonToggleButton ParentUI, ClonedChild;
        
        internal bool IsCloned = false;
        private bool isCloneable = false, isClonedItem = false;
        #endregion
        #region IRibbonControl Implementation
        public bool IsLarge
        {
            get { return this.Size == RibbonControlSize.Large; }
        }

        public Interfaces.IRibbonControl Clone()
        {
            RibbonToggleButton clonedItem = null;
            if (this.ClonedChild == null)
            {
                clonedItem = new RibbonToggleButton();
                clonedItem.ParentUI = this;
                clonedItem.ShouldCreatePopup = false;
                clonedItem.IsClonedItem = true;
                clonedItem.IsCloneable = false;
                this.SetBindings(clonedItem);
                //CloningService.Add(clonedItem);
               
                this.ClonedChild = clonedItem;
            }
            return clonedItem;
        }
        
        public bool IsClonedItem
        {
            get { return this.isClonedItem; }
            internal set { this.isClonedItem = value; }
        }

        public bool IsCloneable
        {
            get 
            {
                return !(this.ClonedChild != null || !RibbonControlCloningService.CheckIfItemIsCloneable(this));
            }
            internal set {this.isCloneable=value; }
        }

        public void DisposeControl()
        {
            if (this.ClonedChild != null) 
            {
                System.Windows.Data.BindingOperations.ClearAllBindings(this.ClonedChild);
                this.ClonedChild = null;
                System.GC.Collect();
            }
        }
        public Interfaces.IRibbonControl ParentControl
        {
            get { return this.ParentUI; }
        }

        #endregion
    }
}
