﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Masterpiece.Controls.Ribbon2010
{
    public static class RibbonWindowParameters
    {
        public static bool IsVistaCore 
        {
            get 
            {
                return System.Environment.OSVersion.Version.Major >= 6;
            }
        }

        /// <summary>
        /// Gets height of window caption button
        /// </summary>
        public static double WindowCaptionButtonHeight 
        {
            get 
            {
                if (RibbonWindowParameters.IsVistaCore)
                {
                    return System.Windows.SystemParameters.WindowCaptionButtonHeight;
                }
                else 
                {
                    return 17.0D;
                }
            } 
        }

        /// <summary>
        /// Gets width of window caption button
        /// </summary>
        public static double WindowCaptionButtonWidth
        {
            get
            {
                if (RibbonWindowParameters.IsVistaCore)
                {
                    return System.Windows.SystemParameters.WindowCaptionButtonWidth;
                }
                else
                {
                    return 30.0D;
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public static double CaptionHeight
        {
            get
            {
                if (RibbonWindowParameters.IsVistaCore)
                {
                    return System.Windows.SystemParameters.WindowCaptionHeight + System.Windows.SystemParameters.ResizeFrameHorizontalBorderHeight;
                    
                }
                else
                {
                    return System.Windows.SystemParameters.WindowCaptionHeight+3;
                }
            }
        }
        
        /// <summary>
        /// Gets margin for window content.
        /// </summary>
        private static System.Windows.Thickness _contentMargin = new System.Windows.Thickness
            (
                //left
                RibbonWindowParameters.BorderWidth, 
                //top
                -1.0D,
                //right
                RibbonWindowParameters.BorderWidth, 
                //bottom
                RibbonWindowParameters.BorderWidth
            );

        /// <summary>
        /// Gets margin for window content.
        /// </summary>
        public static System.Windows.Thickness ContentMargin 
        {
            get 
            {
                return RibbonWindowParameters._contentMargin;
            } 
        }
        /// <summary>
        /// 
        /// </summary>
        public static double BorderWidth 
        {
            get 
            {
                if (RibbonWindowParameters.IsVistaCore && Common.NativeMethods.DwmIsCompositionEnabled()) 
                {
                    return System.Windows.SystemParameters.ResizeFrameVerticalBorderWidth;
                }
                if (RibbonWindowParameters.IsVistaCore)
                {
                    return 5.0D;
                }
                else
                {
                    return 2.0D;
                }
            } 
        }

        /// <summary>
        /// 
        /// </summary>
        private static System.Windows.Thickness _ribbonBarMargin = new System.Windows.Thickness(RibbonWindowParameters.BorderWidth, 0.0D, RibbonWindowParameters.BorderWidth, 0.0D);
        
        /// <summary>
        /// 
        /// </summary>
        public static System.Windows.Thickness RibbonBarMargin
        {
            get
            {
                return RibbonWindowParameters._ribbonBarMargin;
            }
        }

        /// <summary>
        /// Gets content margin when application menu opened. This property used in trigger.
        /// </summary>
        private static System.Windows.Thickness _menuOpenedStateMargin = new System.Windows.Thickness
            (
                //left
                0.0, 
                //top
                23.0D,
                //right
                0.0D, 
                //bottom
                0.0D
            );
        
        /// <summary>
        /// Gets content margin when application menu opened. This property used in trigger.
        /// </summary>
        public static System.Windows.Thickness MenuOpenedStateMargin 
        {
            get 
            {
                return _menuOpenedStateMargin;
            }
        }

        /// <summary>
        /// Gets window icon margin.
        /// </summary>
        private static System.Windows.Thickness _iconMargin = new System.Windows.Thickness(RibbonWindowParameters.BorderWidth, 0, 0, 4);

        /// <summary>
        /// Gets window icon margin.
        /// </summary>
        public static System.Windows.Thickness IconMargin
        {
            get
            {
                return _iconMargin;
            }
        }
        /// <summary>
        /// What does "_XPwbm" mean? Obviously, prefix _XP means if OS version lower or equals to Microsoft Windows XP.
        /// Prefix "wbm" stays for Window Icon Margin. This property is used when window state is maximized.
        /// </summary>
        private static System.Windows.Thickness _XPwbm = new System.Windows.Thickness(0, 3, 5, 0);

        /// <summary>
        /// What does "_VistaCorewim" mean? Obviously, prefix _VistaCore means if OS version larger or equals 
        /// to Microsoft Windows Vista. Prefix "wbm" stays for Window Button Margin. This property is used when window state is maximized.
        /// </summary>
        private static System.Windows.Thickness _VistaCorewbm = new System.Windows.Thickness(0, 7, 5, 0);
 
        /// <summary>
        /// Gets window button margin. This property is used when window is maximized.
        /// </summary>
        public static System.Windows.Thickness MaximizedStateWindowButtonsMargin 
        {
            get 
            {
                if (RibbonWindowParameters.IsVistaCore)
                {
                    return RibbonWindowParameters._VistaCorewbm;
                }
                else 
                {
                    return RibbonWindowParameters._XPwbm;
                }
            }
        }

        /// <summary>
        /// What does "_XPim" mean? Obviously, prefix _XP means if OS version lower or equals to Microsoft Windows XP.
        /// Prefix "im" stays for Icon Margin. This property is used when window state is maximized.
        /// </summary>
        private static System.Windows.Thickness _XPim = new System.Windows.Thickness(3, 5, 0, 0);

        /// <summary>
        /// 
        /// </summary>
        public static System.Windows.Thickness MaximizedStateIconMargin
        {
            get
            {
                return RibbonWindowParameters._XPim;
            }
        }

        /// <summary>
        /// Gets footer height for ribbon window.
        /// </summary>
        private static double _footerHeight = 23.0;
        
        /// <summary>
        /// Gets footer height for ribbon window. 
        /// </summary>
        public static double FooterHeight
        {
            get
            {
                return RibbonWindowParameters._footerHeight;
            }
        }

        /// <summary>
        /// Gets content paddind of window.
        /// </summary>
       private static System.Windows.Thickness _contentPadding = new System.Windows.Thickness(0.0D);

       public static System.Windows.Thickness ContentPadding 
       {
           get 
           {
               return RibbonWindowParameters._contentPadding;
           } 
       }
    }
}
