﻿
namespace Masterpiece.Controls.Ribbon2010
{
    internal class TwoLineText:System.Windows.Controls.Control
    {

        #region Constructors
        
        static TwoLineText()
        {
            System.Type ownerType = typeof(TwoLineText);
            TwoLineText.DefaultStyleKeyProperty.OverrideMetadata(ownerType, new System.Windows.FrameworkPropertyMetadata(ownerType));
        }
        
        #endregion

        #region Properties and Depenency Properties

        internal string Text
        {
            get { return (string)GetValue(TextProperty); }
            set { SetValue(TextProperty, value); }
        }

        // Using a DependencyProperty as the backing store for Text.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty TextProperty =
            System.Windows.DependencyProperty.Register("Text", typeof(string), typeof(TwoLineText), new System.Windows.FrameworkPropertyMetadata(
                (string)null, 
                System.Windows.FrameworkPropertyMetadataOptions.BindsTwoWayByDefault, 
                new System.Windows.PropertyChangedCallback(TwoLineText.OnTextPropertyChanged)));

        internal string Text1
        {
            get { return (string)GetValue(Text1Property); }
            set { SetValue(Text1Property, value); }
        }

        // Using a DependencyProperty as the backing store for Text1.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty Text1Property =
            System.Windows.DependencyProperty.Register("Text1", typeof(string), typeof(TwoLineText), new System.Windows.FrameworkPropertyMetadata(
                (string)null,
                System.Windows.FrameworkPropertyMetadataOptions.AffectsMeasure));

        internal string Text2
        {
            get { return (string)GetValue(Text2Property); }
            set { SetValue(Text2Property, value); }
        }

        // Using a DependencyProperty as the backing store for Text2.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty Text2Property =
            System.Windows.DependencyProperty.Register("Text2", typeof(string), typeof(TwoLineText), new System.Windows.FrameworkPropertyMetadata((string)null, 
                System.Windows.FrameworkPropertyMetadataOptions.AffectsMeasure));

        internal System.Windows.Visibility ImageVisibility
        {
            get { return (System.Windows.Visibility)GetValue(ImageVisibilityProperty); }
            set { SetValue(ImageVisibilityProperty, value); }
        }

        // Using a DependencyProperty as the backing store for ImageVisibility.  This enables animation, styling, binding, etc...
        internal static readonly System.Windows.DependencyProperty ImageVisibilityProperty =
            System.Windows.DependencyProperty.Register("ImageVisibility", typeof(System.Windows.Visibility), typeof(TwoLineText), new System.Windows.PropertyMetadata(System.Windows.Visibility.Collapsed));

        #endregion

        #region Depenency Property Changed Callbacks

        private static void OnTextPropertyChanged(System.Windows.DependencyObject d, System.Windows.DependencyPropertyChangedEventArgs e) 
        {
            
            string text = (string)e.NewValue;
            TwoLineText twoLine = (TwoLineText)d;
            if (!System.String.IsNullOrEmpty(text))
            {
                twoLine.SetText();
            }
            else 
            {
                twoLine.Text1 = System.String.Empty;
                twoLine.Text2 = System.String.Empty;
            }

        }
        #endregion

        #region Overrides
        public override void OnApplyTemplate()
        {
            base.OnApplyTemplate();
            this.TextBlock1 = this.GetTemplateChild("PART_TwoLineText1") as System.Windows.Controls.TextBlock;
            this.TextBlock2 = this.GetTemplateChild("PART_TwoLineText2") as System.Windows.Controls.TextBlock;
        }

        #endregion

        #region Private Utilities

        private void SetText() 
        {
            if (this.Text.Contains(@"\n"))
            {
                int index = this.Text.IndexOf(@"\n");
                string text1 = this.Text.Substring(0, index);
                string text2 = this.Text.Remove(0, text1.Length + 2);
                text1 = text1.Trim();
                text2 = text2.Trim();
                if (!System.String.IsNullOrEmpty(text1) || !System.String.IsNullOrEmpty(text2))
                {
                    if (System.String.IsNullOrEmpty(text1) && !System.String.IsNullOrEmpty(text2))
                    {
                        this.Text1 = text2;
                    }
                    else if (!System.String.IsNullOrEmpty(text1))
                    {
                        this.Text1 = text1;
                        this.Text2 = !System.String.IsNullOrEmpty(text2) ? text2 : null;
                    }
                }
            }
            else 
            {
                this.Text1 = this.Text;
            }
        }

        private void SetMargin() 
        {
            if (this.TextBlock1 != null && this.TextBlock2 != null) 
            {
                double largeScale = this.TextBlock1.DesiredSize.Width > this.TextBlock2.DesiredSize.Width ? this.TextBlock1.DesiredSize.Width : this.TextBlock2.DesiredSize.Width;
            }
        }
        #endregion

        #region Event Handlers

        #endregion

        #region Fields
        private System.Windows.Controls.TextBlock TextBlock1, TextBlock2;
        
        #endregion

        #region Events
        #endregion
    }
}
